
\name{e04jy}
\alias{e04jy}

\title{e04jy: Minimum, function of several variables, quasi-Newton algorithm, simple 
bounds, using function values only (easy-to-use)}

\description{
e04jy is an easy-to-use quasi-Newton algorithm for finding a minimum of 
a function \eqn{{F}({x}_{{1}}{x}_{{2}} \ldots {x}_{{n}})}{F(x_1x_2 . . . x_n)}, 
subject to fixed upper and lower bounds of the independent variables \eqn{{x}_{{1}} 
, {x}_{{2}} , \ldots , {x}_{{n}}}{x_1 , x_2 , . . . , x_n}, using function values 
only.

It is intended for functions which are continuous and which have continuous 
first and second derivatives (although it will usually work even if the 
derivatives have occasional discontinuities).

}

\usage{
e04jy(ibound, funct1, bl, bu, x,
n=nrow(bl),
liw=n+2,
lw=max(n*(n-1)/2+12*n,13)
)
}

\arguments{


\item{ibound}{integer

Indicates whether the facility for dealing with bounds of special forms 
is to be used.


}
\item{funct1}{void function

You must supply funct1 to calculate the value of the function \eqn{{F}({x})}{F(x)} at any point \eqn{{x}}{x}.  It should be tested separately before being used with e04jy (see the E04 chapter introduction in the Fortran Library documentation).
}
\item{bl}{double array

The lower bounds \eqn{{l}_{{j}}}{l_j}.


}
\item{bu}{double array

The upper bounds \eqn{{u}_{{j}}}{u_j}.


}
\item{x}{double array

\eqn{x({j})}{x(j)} must be set to an estimate of the \eqn{{j}}{j}th component 
of the position of the minimum for \eqn{{j}={1} \ldots {n}}{j=1 . . . n}.


}
\item{n}{integer: \strong{default} = nrow(bl)

The number \eqn{{n}}{n} of independent variables.


}
\item{liw}{integer: \strong{default} = n+2

}
\item{lw}{integer: \strong{default} = max(n*(n-1)/2+12*n,13)

}
}

\details{
R interface to the NAG Fortran routine E04JYF.}

\value{

\item{bl}{double array

The lower bounds actually used by e04jy.


}
\item{bu}{double array

The upper bounds actually used by e04jy.


}
\item{x}{double array

The lowest point found during the calculations. Thus, if \eqn{\mathrm{ifail} 
=0}{ifail =0} on exit, \eqn{x({j})}{x(j)} is the \eqn{{j}}{j}th component 
of the position of the minimum.


}
\item{f}{double

The value of \eqn{{F}({x})}{F(x)} corresponding to the final point stored 
in x.


}
\item{iw}{integer array

If \eqn{\mathrm{ifail} =0}{ifail =0}, \eqn{\mathrm{ifail} =3}{ifail =3}, 
\eqn{\mathrm{ifail} =5}{ifail =5}, the first n elements of iw contain information 
about which variables are currently on their bounds and which are free. 
Specifically, if \eqn{{x}_{{i}}}{x_i} is: 

-: fixed on its upper bound, \eqn{iw({i})}{iw(i)} is \eqn{{ - {1}}}{ - 
1};

-: fixed on its lower bound, \eqn{iw({i})}{iw(i)} is \eqn{{ - {2}}}{ - 
2};

-: effectively a constant (i.e., \eqn{{l}_{{j}} = {u}_{{j}}}{l_j = u_j}), 
\eqn{iw({i})}{iw(i)} is \eqn{{ - {3}}}{ - 3};

-: free, \eqn{iw({i})}{iw(i)} gives its position in the sequence of free 
variables.




}
\item{w}{double array

If \eqn{\mathrm{ifail} =0}{ifail =0}, \eqn{\mathrm{ifail} =3}{ifail =3}, 
\eqn{\mathrm{ifail} =5}{ifail =5}, \eqn{w({i})}{w(i)} contains a finite 
difference approximation to the \eqn{{i}}{i}th element of the projected 
gradient vector \eqn{{g}_{{z}}}{g_z} for \eqn{{i}={1} \ldots n}{i=1 . . . n}. 
In addition, \eqn{w(n+{1})}{w(n+1)} contains an estimate of the condition 
number of the projected Hessian matrix (i.e., \eqn{{k}}{k}). The rest of 
the array is used as workspace.


}}

\references{
\url{http://www.nag.co.uk/numeric/FL/nagdoc_fl23/pdf/E04/e04jyf.pdf}
}

\author{NAG}

\examples{
e04jy_funct1 = function(n, xc, fc) {
    
    fc <- (xc[1] + 10 * xc[2])^2 + 5 * (xc[3] - xc[4])^2 + (xc[2] - 
        2 * xc[3])^4 + 10 * (xc[1] - xc[4])^4
    list(FC = fc)
}

ibound <- 0
bl <- matrix(c(1, -2, -1e+06, 1), nrow = 4, ncol = 1, 
    byrow = TRUE)


bu <- matrix(c(3, 0, 1e+06, 3), nrow = 4, ncol = 1, 
    byrow = TRUE)


x <- matrix(c(3, -1, 0, 1), nrow = 4, ncol = 1, byrow = TRUE)


e04jy(ibound, e04jy_funct1, bl, bu, x) 

}

\keyword{math}

\keyword{optimize}
\concept{bound}
\concept{minimum}
\concept{quasi-Newton}
