
\name{e04ky}
\alias{e04ky}

\title{e04ky: Minimum, function of several variables, quasi-Newton algorithm, simple 
bounds, using first derivatives (easy-to-use)}

\description{
e04ky is an easy-to-use quasi-Newton algorithm for finding a minimum of 
a function \eqn{{F}\left({x}_{{1}}{x}_{{2}} \ldots {x}_{{n}}\right)}{F(x_1x_2  . . .  x_n)}, 
subject to fixed upper and lower bounds on the independent variables \eqn{{x}_{{1}} 
, {x}_{{2}} , \ldots , {x}_{{n}}}{x_1 , x_2 ,  . . .  , x_n}, when first derivatives 
of \eqn{{F}}{F} are available.

It is intended for functions which are continuous and which have continuous 
first and second derivatives (although it will usually work even if the 
derivatives have occasional discontinuities).

}

\usage{
e04ky(ibound, funct2, bl, bu, x,
      n = nrow(bl),
      liw = (n+2),
      lw = (max((10*n+n*(n-1)/2),11)))
}

\arguments{

\item{ibound}{integer

Indicates whether the facility for dealing with bounds of special forms 
is to be used. It must be set to one of the following values: 

\eqn{ibound = {0}}{ibound = 0}: If you are supplying all the \eqn{{l}_{{j}}}{l_j} 
and \eqn{{u}_{{j}}}{u_j} individually.

\eqn{ibound = {1}}{ibound = 1}: If there are no bounds on any \eqn{{x}_{{j}}}{x_j}.

\eqn{ibound 
= {2}}{ibound = 2}: If all the bounds are of the form \eqn{{0} \leq {x}_{{j}}}{0  <=  x_j}.

\eqn{ibound 
= {3}}{ibound = 3}: If \eqn{{l}_{{1}} = {l}_{{2}} = \cdots = {l}_{{n}}}{l_1 = l_2 =  . . .  = l_n} 
and \eqn{{u}_{{1}} = {u}_{{2}} = \cdots = {u}_{{n}}}{u_1 = u_2 =  . . .  = u_n}.




}
\item{funct2}{function

You must supply funct2 to calculate the values of the function \eqn{{F}\left({x}\right)}{F(x)} and its first derivative \eqn{\frac{{ \partial {F}}}{{ \partial {x}_{{j}}}}}{( \partial F)/( \partial x_j)} at any point \eqn{{x}}{x}.  It should be tested separately before being used in conjunction with e04ky (see the E04 chapter introduction in the Fortran Library documentation).

\code{(FC,GC) = funct2(n,xc)
}


}
\item{bl}{double array

The lower bounds \eqn{{l}_{{j}}}{l_j}.


}
\item{bu}{double array

The upper bounds \eqn{{u}_{{j}}}{u_j}.


}
\item{x}{double array

\eqn{x[{j}]}{x[j]} must be set to a guess at the \eqn{{j}}{j}th component 
of the position of the minimum for \eqn{{j}={1} \ldots {n}}{j=1  . . .  n}. 
The function checks the gradient at the starting point, and is more likely 
to detect any error in your programming if the initial \eqn{x[{j}]}{x[j]} 
are nonzero and mutually distinct.


}
\item{n}{integer: \strong{default} = nrow(bl)

The number \eqn{{n}}{n} of independent variables.


}
\item{liw}{integer: \strong{default} = (n+2)




}
\item{lw}{integer: \strong{default} = (max((10*n+n*(n-1)/2),11))




}
}

\details{
R interface to the NAG Fortran routine E04KYF.}

\value{

\item{BL}{double array

The lower bounds actually used by e04ky.


}
\item{BU}{double array

The upper bounds actually used by e04ky.


}
\item{X}{double array

The lowest point found during the calculations. Thus, if \eqn{\mathrm{ifail} 
=0}{ifail =0} on exit, \eqn{x[{j}]}{x[j]} is the \eqn{{j}}{j}th component 
of the position of the minimum.


}
\item{F}{double

The value of \eqn{{F}\left({x}\right)}{F(x)} corresponding to the final 
point stored in x.


}
\item{G}{double array

The value of \eqn{\frac{{ \partial {F}}}{{ \partial {x}_{{j}}}}}{( \partial F)/( \partial x_j)} 
corresponding to the final point stored in x for \eqn{{j}={1} \ldots {n}}{j=1  . . .  n}; 
the value of \eqn{g[{j}]}{g[j]} for variables not on a bound should normally 
be close to zero.


}
\item{IW}{integer array

If \eqn{\mathrm{ifail} =0}{ifail =0}, \eqn{\mathrm{ifail} =3}{ifail =3}, 
\eqn{\mathrm{ifail} =5}{ifail =5}, the first n elements of iw contain information 
about which variables are currently on their bounds and which are free. 
Specifically, if \eqn{{x}_{{i}}}{x_i} is: 

-: fixed on its upper bound, \eqn{iw[{i}]}{iw[i]} is \eqn{{ - {1}}}{ - 1};

-: 
fixed on its lower bound, \eqn{iw[{i}]}{iw[i]} is \eqn{{ - {2}}}{ - 2};

-: 
effectively a constant (i.e., \eqn{{l}_{{j}} = {u}_{{j}}}{l_j = u_j}), 
\eqn{iw[{i}]}{iw[i]} is \eqn{{ - {3}}}{ - 3};

-: free, \eqn{iw[{i}]}{iw[i]} gives its position in the sequence of free 
variables.




}
\item{W}{double array

If \eqn{\mathrm{ifail} =0}{ifail =0}, \eqn{\mathrm{ifail} =3}{ifail =3}, 
\eqn{\mathrm{ifail} =5}{ifail =5}, \eqn{w[{i}]}{w[i]} contains the \eqn{{i}}{i}th 
element of the projected gradient vector \eqn{{g}_{{z}}}{g_z} for \eqn{{i}={1} 
\ldots n}{i=1  . . .  n}. In addition, \eqn{w[n+{1}]}{w[n+1]} contains an estimate 
of the condition number of the projected Hessian matrix (i.e., \eqn{{k}}{k}). 
The rest of the array is used as workspace.


}
\item{IFAIL}{integer

\eqn{\mathrm{ifail} =0}{ifail =0} unless the function detects an error 
or a warning has been flagged (see the Errors section in Fortran library 
documentation).


}
}

\references{
\url{http://www.nag.co.uk/numeric/FL/nagdoc_fl23/pdf/E04/e04kyf.pdf}
}

\author{NAG}

\examples{

ifail<-0
funct2=function(n,xc){

gc<-as.matrix(mat.or.vec(n,1))
fc<-(xc[1]+10\%*\%xc[2])^2+5\%*\%(xc[3]-xc[4])^2+(xc[2]-2\%*\%xc[3])^4+10\%*\%(xc[1]-xc[4])^4
gc[1]<-2\%*\%(xc[1]+10\%*\%xc[2])+40\%*\%(xc[1]-xc[4])^3
gc[2]<-20\%*\%(xc[1]+10\%*\%xc[2])+4\%*\%(xc[2]-2\%*\%xc[3])^3
gc[3]<-10\%*\%(xc[3]-xc[4])-8\%*\%(xc[2]-2\%*\%xc[3])^3
gc[4]<--10\%*\%(xc[3]-xc[4])-40\%*\%(xc[1]-xc[4])^3
list(FC=fc,GC=as.matrix(gc))
}

ibound<-0

bl<-matrix(c(1,-2,-1000000,1),nrow=4,ncol=1,byrow=TRUE)



bu<-matrix(c(3,0,1000000,3),nrow=4,ncol=1,byrow=TRUE)



x<-matrix(c(3,-1,0,1),nrow=4,ncol=1,byrow=TRUE)



e04ky(ibound,funct2,bl,bu,x)

}

\keyword{math}

\keyword{optimize}
\concept{bound}
\concept{derivative}
\concept{minimum}
\concept{quasi-Newton}
