
\name{e04nk}
\alias{e04nk}

\title{e04nk: LP or QP problem (sparse)}

\description{
e04nk solves sparse linear programming or quadratic programming problems.


}

\usage{
e04nk(n, m, iobj, ncolh, qphx, a, ha, ka, bl, bu, start, names, crname, ns, xs, istate, leniz, lenz, optlist,
      nnz = nrow(a),
      nname = nrow(crname))
}

\arguments{

\item{n}{integer

\eqn{{n}}{n}, the number of variables (excluding slacks). This is the number 
of columns in the linear constraint matrix \eqn{{A}}{A}.


}
\item{m}{integer

\eqn{{m}}{m}, the number of general linear constraints (or slacks). This 
is the number of rows in \eqn{{A}}{A}, including the free row (if any; 
see iobj).


}
\item{iobj}{integer

If \eqn{iobj > {0}}{iobj > 0}, row iobj of \eqn{{A}}{A} is a free row containing 
the nonzero elements of the vector \eqn{{c}}{c} appearing in the linear 
objective term \eqn{{c}^{T}{x}}{c^Tx}.


}
\item{ncolh}{integer

\eqn{{n}_{{H}}}{n_H}, the number of leading nonzero columns of the Hessian 
matrix \eqn{{H}}{H}. For FP and LP problems, ncolh must be set to zero.


}
\item{qphx}{function

For QP problems, you must supply a version of qphx to compute the matrix product \eqn{{H}{x}}{Hx}.  If \eqn{{H}}{H} has zero rows and columns, it is most efficient to order the variables \eqn{{x} = \left(\begin{array}{cc} {y}&{z}\\ \end{array}\right) ^{T}}{x = 
(
y z
)
^T} so that \deqn{{H}{x} = \left(\begin{array}{cccc} {H}_{{1}}&{0}\\ {0}&{0}\\ \end{array}\right) \left(\begin{array}{cc} {y}\\ {z}\\ \end{array}\right) = \left(\begin{array}{cc} {H}_{{1}}{y}\\ {0}\\ \end{array}\right) \mbox{,}}{Hx = 
(
H_1 0
0 0
)

(
y
z
)
 = 
(
H_1y
0
)
,} where the nonlinear variables \eqn{{y}}{y} appear first as shown.  For FP and LP problems, qphx will never be called by e04nk and hence qphx may be the dummy function e04nku.

\code{(HX) = qphx(nstate,ncolh,x)
}


}
\item{a}{double array

The nonzero elements of \eqn{{A}}{A}, ordered by increasing column index. 
Note that elements with the same row and column indices are not allowed.


}
\item{ha}{integer array

\eqn{ha[{i}]}{ha[i]} must contain the row index of the nonzero element 
stored in \eqn{a[{i}]}{a[i]} for \eqn{{i}={1} \ldots nnz}{i=1  . . .  nnz}. 
Note that the row indices for a column may be supplied in any order.


}
\item{ka}{integer array

\eqn{ka[{j}]}{ka[j]} must contain the index in a of the start of the \eqn{{j}}{j}th 
column for \eqn{{j}={1} \ldots n}{j=1  . . .  n}. To specify the \eqn{{j}}{j}th 
column as empty, set \eqn{ka[{j}] = ka[{j}+{1}]}{ka[j] = ka[j+1]}. Note 
that the first and last elements of ka must be such that \eqn{ka[{1}] = 
{1}}{ka[1] = 1} and \eqn{ka[n+{1}] = nnz + {1}}{ka[n+1] = nnz + 1}.


}
\item{bl}{double array

\eqn{{l}}{l}, the lower bounds for all the variables and general constraints, 
in the following order. The first n elements of bl must contain the bounds 
on the variables \eqn{{x}}{x}, and the next m elements the bounds for the 
general linear constraints \eqn{{A}{x}}{Ax} (or slacks \eqn{{s}}{s}) and 
the free row (if any). To specify a nonexistent lower bound (i.e., \eqn{{l}_{{j}} 
= { - {\infty}}}{l_j =  -  infinity}), set \eqn{bl[{j}] \leq - {bigbnd}}{bl[j]  <=  - bigbnd}, 
where \eqn{{bigbnd}}{bigbnd} is the value of the optional argument infiniteboundsize. 
To specify the \eqn{{j}}{j}th constraint as an \emph{equality}, set \eqn{bl[{j}] 
= bu[{j}] = {\beta}}{bl[j] = bu[j] = \beta}, say, where \eqn{\mathrm{abs}({\beta}) 
< {bigbnd}}{abs(\beta) < bigbnd}. Note that the lower bound corresponding to 
the free row must be set to \eqn{{ - {\infty}}}{ -  infinity} and stored in 
\eqn{bl[n+iobj]}{bl[n+iobj]}.


}
\item{bu}{double array

\eqn{{u}}{u}, the upper bounds for all the variables and general constraints, 
in the following order. The first n elements of bu must contain the bounds 
on the variables \eqn{{x}}{x}, and the next m elements the bounds for the 
general linear constraints \eqn{{A}{x}}{Ax} (or slacks \eqn{{s}}{s}) and 
the free row (if any). To specify a nonexistent upper bound (i.e., \eqn{{u}_{{j}} 
= { + {\infty}}}{u_j =  +  infinity}), set \eqn{bu[{j}] \geq {bigbnd}}{bu[j]  >=  bigbnd}. 
Note that the upper bound corresponding to the free row must be set to 
\eqn{{ + {\infty}}}{ +  infinity} and stored in \eqn{bu[n+iobj]}{bu[n+iobj]}.


}
\item{start}{string

Indicates how a starting basis is to be obtained. 

\eqn{start=\mathtt{'C'}}{start='C'}: An internal Crash procedure will be 
used to choose an initial basis matrix \eqn{{B}}{B}.

\eqn{start=\mathtt{'W'}}{start='W'}: A basis is already defined in istate 
(probably from a previous call).




}
\item{names}{string array

A set of names associated with the so-called MPSX form of the problem, 
as follows: 

\eqn{names[{1}]}{names[1]}: Must contain the name for the problem (or be 
blank).

\eqn{names[{2}]}{names[2]}: Must contain the name for the free row (or 
be blank).

\eqn{names[{3}]}{names[3]}: Must contain the name for the constraint right-hand 
side (or be blank).

\eqn{names[{4}]}{names[4]}: Must contain the name for the ranges (or be 
blank).

\eqn{names[{5}]}{names[5]}: Must contain the name for the bounds (or be 
blank).




}
\item{crname}{string array

The optional column and row names, respectively.


}
\item{ns}{integer

\eqn{{n}_{{S}}}{n_S}, the number of superbasics. For QP problems, ns need 
not be specified if \eqn{start=\mathtt{'C'}}{start='C'}, but must retain 
its value from a previous call when \eqn{start=\mathtt{'W'}}{start='W'}. 
For FP and LP problems, ns need not be initialized.


}
\item{xs}{double array

The initial values of the variables and slacks \eqn{\left({x}{s}\right)}{(xs)}. 
(See the description for istate.)


}
\item{istate}{integer array

If \eqn{start=\mathtt{'C'}}{start='C'}, the first n elements of istate 
and xs must specify the initial states and values, respectively, of the 
variables \eqn{{x}}{x}. (The slacks \eqn{{s}}{s} need not be initialized.) 
An internal Crash procedure is then used to select an initial basis matrix 
\eqn{{B}}{B}. The initial basis matrix will be triangular (neglecting certain 
small elements in each column). It is chosen from various rows and columns 
of \eqn{\left(\begin{array}{cc} {A}&-{I}\\ \end{array}\right)}{
(
A -I
)
}. Possible values for \eqn{istate[{j}]}{istate[j]} are as follows:


}
\item{leniz}{integer




}
\item{lenz}{integer




}
\item{optlist}{options list

Optional parameters may be listed, as shown in the following table:

\tabular{lll}{
\bold{Name} \tab \bold{Type} \tab \bold{Default} \cr
\code{Check Frequency} \tab \emph{integer} \tab Default \eqn{\mbox{} = {60}}{ = 60} \cr
\code{Crash Option} \tab \emph{integer} \tab Default \eqn{\mbox{} = {2}}{ = 2} \cr
\code{Crash Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = {0.1}}{ = 0.1} \cr
\code{Defaults } \tab \emph{} \tab  \cr
\code{Expand Frequency} \tab \emph{integer} \tab Default \eqn{\mbox{} = {10000}}{ = 10000} \cr
\code{Factorization Frequency} \tab \emph{integer} \tab Default \eqn{\mbox{} = {100}}{ = 100} \cr
\code{Feasibility Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = \mathrm{max}({10}^{{ - {6}}}, {\sqrt{{\epsilon}}})}{ = max(10^ - 6, sqrt(\epsilon))} \cr
\code{Infinite Bound Size} \tab \emph{double} \tab Default \eqn{\mbox{} = {10}^{{20}}}{ = 10^20} \cr
\code{Infinite Step Size} \tab \emph{double} \tab Default \eqn{\mbox{} = \mathrm{max}({bigbnd}, {10}^{{20}})}{ = max(bigbnd, 10^20)} \cr
\code{Iteration Limit} \tab \emph{integer} \tab Default \eqn{\mbox{} = \mathrm{max}({50}, {{5}\left({n} + {m}\right)})}{ = max(50, 5(n + m))} \cr
\code{Iters} \tab \emph{} \tab  \cr
\code{Itns} \tab \emph{} \tab  \cr
\code{List} \tab \emph{} \tab Default for \eqn{e04nk = list}{e04nk = list} \cr
\code{Nolist} \tab \emph{} \tab Default for \eqn{e04nk = nolist}{e04nk = nolist} \cr
\code{LU Factor Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = {100.0}}{ = 100.0} \cr
\code{LU Update Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = {10.0}}{ = 10.0} \cr
\code{LU Singularity Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = {\epsilon}^{{0.67}}}{ = \epsilon^0.67} \cr
\code{Minimize} \tab \emph{} \tab Default \cr
\code{Maximize} \tab \emph{} \tab  \cr
\code{Monitoring File} \tab \emph{integer} \tab Default \eqn{\mbox{} = { - {1}}}{ =  - 1} \cr
\code{Optimality Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = \mathrm{max}({10}^{{ - {6}}}, {\sqrt{{\epsilon}}})}{ = max(10^ - 6, sqrt(\epsilon))} \cr
\code{Partial Price} \tab \emph{integer} \tab Default \eqn{\mbox{} = {10}}{ = 10} \cr
\code{Pivot Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = {\epsilon}^{{0.67}}}{ = \epsilon^0.67} \cr
\code{Print Level} \tab \emph{integer} \tab \eqn{\mbox{} = {0}}{ = 0} \cr
\code{Rank Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = {100}{\epsilon}}{ = 100\epsilon} \cr
\code{Scale Option} \tab \emph{integer} \tab Default \eqn{\mbox{} = {2}}{ = 2} \cr
\code{Scale Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = {0.9}}{ = 0.9} \cr
\code{Superbasics Limit} \tab \emph{integer} \tab Default \eqn{\mbox{} = \mathrm{min}({{n}_{{H}} + {1}}, {n})}{ = min(n_H + 1, n)} \cr
}
}
\item{nnz}{integer: \strong{default} = nrow(a)

The number of nonzero elements in \eqn{{A}}{A}.


}
\item{nname}{integer: \strong{default} = nrow(crname)

The number of column (i.e., variable) and row names supplied in crname. 


\eqn{nname = {1}}{nname = 1}: There are no names. Default names will be 
used in the printed output.

\eqn{nname = n + m}{nname = n + m}: All names must be supplied.




}
}

\details{
R interface to the NAG Fortran routine E04NKF.}

\value{

\item{NS}{integer

The final number of superbasics. This will be zero for FP and LP problems.


}
\item{XS}{double array

The final values of the variables and slacks \eqn{\left({x}{s}\right)}{(xs)}.


}
\item{ISTATE}{integer array

The final states of the variables and slacks \eqn{\left({x}{s}\right)}{(xs)}. 
The significance of each possible value of \eqn{istate[{j}]}{istate[j]} 
is as follows:


}
\item{MINIZ}{integer

The minimum value of leniz required to start solving the problem. If \eqn{\mathrm{ifail} 
=12}{ifail =12}, e04nk may be called again with leniz suitably larger 
than miniz. (The bigger the better, since it is not certain how much workspace 
the basis factors need.)


}
\item{MINZ}{integer

The minimum value of lenz required to start solving the problem. If \eqn{\mathrm{ifail} 
=13}{ifail =13}, e04nk may be called again with lenz suitably larger than 
minz. (The bigger the better, since it is not certain how much workspace 
the basis factors need.)


}
\item{NINF}{integer

The number of infeasibilities. This will be zero if \eqn{\mathrm{ifail} 
=0}{ifail =0}, \eqn{\mathrm{ifail} =1}{ifail =1}.


}
\item{SINF}{double

The sum of infeasibilities. This will be zero if \eqn{ninf = {0}}{ninf = 0}. 
(Note that e04nk does \emph{not} attempt to compute the minimum value 
of sinf if \eqn{\mathrm{ifail} =3}{ifail =3}.)


}
\item{OBJ}{double

The value of the objective function.


}
\item{CLAMDA}{double array

A set of Lagrange multipliers for the bounds on the variables and the general 
constraints. More precisely, the first n elements contain the multipliers 
(\emph{reduced costs}) for the bounds on the variables, and the next m 
elements contain the multipliers (\emph{shadow prices}) for the general 
linear constraints.


}
\item{IFAIL}{integer

\eqn{\mathrm{ifail} =0}{ifail =0} unless the function detects an error 
or a warning has been flagged (see the Errors section in Fortran library 
documentation).


}
}

\references{
\url{http://www.nag.co.uk/numeric/FL/nagdoc_fl23/pdf/E04/e04nkf.pdf}
}

\author{NAG}

\examples{

optlist <- list()

ifail <- 0
qphx = function(nstate, ncolh, x) {
    
    hx <- as.matrix(mat.or.vec(ncolh, 1))
    hx[1] <- 2 \%*\% x[1]
    hx[2] <- 2 \%*\% x[2]
    hx[3] <- 2 \%*\% (x[3] + x[4])
    hx[4] <- hx[3]
    hx[5] <- 2 \%*\% x[5]
    hx[6] <- 2 \%*\% (x[6] + x[7])
    hx[7] <- hx[6]
    list(HX = as.matrix(hx))
}

n <- 7

m <- 8

iobj <- 8

ncolh <- 7

a <- matrix(c(0.02, 0.02, 0.03, 1, 0.7, 0.02, 0.15, 
    -200, 0.06, 0.75, 0.03, 0.04, 0.05, 0.04, 1, -2000, 0.02, 
    1, 0.01, 0.08, 0.08, 0.8, -2000, 1, 0.12, 0.02, 0.02, 0.75, 
    0.04, -2000, 0.01, 0.8, 0.02, 1, 0.02, 0.06, 0.02, -2000, 
    1, 0.01, 0.01, 0.97, 0.01, 400, 0.97, 0.03, 1, 400), nrow = 48, 
    ncol = 1, byrow = TRUE)



ha <- matrix(c(7, 5, 3, 1, 6, 4, 2, 8, 7, 6, 5, 4, 
    3, 2, 1, 8, 2, 1, 4, 3, 7, 6, 8, 1, 7, 3, 4, 6, 2, 8, 5, 
    6, 7, 1, 2, 3, 4, 8, 1, 2, 3, 6, 7, 8, 7, 2, 1, 8), nrow = 48, 
    ncol = 1, byrow = TRUE)



ka <- matrix(c(1, 9, 17, 24, 31, 39, 45, 49), nrow = 8, 
    ncol = 1, byrow = TRUE)



bl <- matrix(c(0, 0, 400, 100, 0, 0, 0, 2000, -1e+25, 
    -1e+25, -1e+25, -1e+25, 1500, 250, -1e+25), nrow = 15, ncol = 1, 
    byrow = TRUE)



bu <- matrix(c(200, 2500, 800, 700, 1500, 1e+25, 1e+25, 
    2000, 60, 100, 40, 30, 1e+25, 300, 1e+25), nrow = 15, ncol = 1, 
    byrow = TRUE)



start <- "C"

names <- matrix(c("        ", "        ", "        ", 
    "        ", "        "), nrow = 5, byrow = TRUE)



crname <- matrix(c("...X1...", "...X2...", "...X3...", 
    "...X4...", "...X5...", "...X6...", "...X7...", "..ROW1..", 
    "..ROW2..", "..ROW3..", "..ROW4..", "..ROW5..", "..ROW6..", 
    "..ROW7..", "..COST.."), nrow = 15, byrow = TRUE)



ns <- -1232765364

xs <- matrix(c(0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
    0, 0, 0), nrow = 15, ncol = 1, byrow = TRUE)



istate <- as.matrix(mat.or.vec(15, 1))

leniz <- 10000

lenz <- 10000

ans <- e04nk(n, m, iobj, ncolh, qphx, a, ha, ka, bl, 
    bu, start, names, crname, ns, xs, istate, leniz, lenz, optlist) 
ans
}

\keyword{math}

\keyword{optimize}
\concept{LP}
\concept{QP}
