
\name{e04nq}
\alias{e04nq}

\title{e04nq: LP or QP problem (suitable for sparse problems)}

\description{
e04nq solves sparse linear programming or convex quadratic programming 
problems. The initialization function e04np \bold{must} have been called 
before calling e04nq.

}

\usage{
e04nq(start, qphx, m, n, lenc, ncolh, iobj, objadd, prob, acol, inda, loca, bl, bu, c, names, helast, hs, x, ns, optlist,
      ne = nrow(acol),
      nname = nrow(names))
}

\arguments{

\item{start}{string

Indicates how a starting basis (and certain other items) will be obtained. 


\eqn{start=\mathtt{'C'}}{start='C'}: Requests that an internal Crash procedure 
be used to choose an initial basis, unless a Basis file is provided via 
optional arguments oldbasisfile, insertfile or loadfile.

\eqn{start=\mathtt{'B'}}{start='B'}: Is the same as \eqn{start=\mathtt{'C'}}{start='C'} 
but is more meaningful when a Basis file is given.

\eqn{start=\mathtt{'W'}}{start='W'}: Means that a basis is already defined 
in hs and a start point is already defined in x (probably from an earlier 
call).




}
\item{qphx}{function

For QP problems, you must supply a version of qphx to compute the matrix product \eqn{{H}{x}}{Hx} for a given vector \eqn{{x}}{x}.  If \eqn{{H}}{H} has rows and columns of zeros, it is most efficient to order \eqn{{x}}{x} so that the nonlinear variables appear first.  For example, if \eqn{{x} = \left({y}{z}\right)^{T}}{x = (yz)^T} and only \eqn{{y}}{y} enters the objective quadratically then \deqn{{H}{x} = \left(\begin{array}{cccc} {H}_{{1}}&{0}\\ {0}&{0}\\ \end{array}\right) \left(\begin{array}{cc} {y}\\ {z}\\ \end{array}\right) = \left(\begin{array}{cc} {H}_{{1}}{y}\\ {0}\\ \end{array}\right) \mbox{.}}{Hx = 
(
H_1 0
0 0
)

(
y
z
)
 = 
(
H_1y
0
)
.} In this case, ncolh should be the dimension of \eqn{{y}}{y}, and qphx should compute \eqn{{H}_{{1}}{y}}{H_1y}. For FP and LP problems, qphx will never be called by e04nq and hence qphx may be 
the dummy function e04nsh.



\code{(HX) = qphx(ncolh,x,nstate)
}


}
\item{m}{integer

\eqn{{m}}{m}, the number of general linear constraints (or slacks). This 
is the number of rows in the linear constraint matrix \eqn{{A}}{A}, including 
the free row (if any; see iobj). Note that \eqn{{A}}{A} must have at least 
one row. If your problem has no constraints, or only upper or lower bounds 
on the variables, then you must include a dummy row with sufficiently wide 
upper and lower bounds (see also acol, inda and loca).


}
\item{n}{integer

\eqn{{n}}{n}, the number of variables (excluding slacks). This is the number 
of columns in the linear constraint matrix \eqn{{A}}{A}.


}
\item{lenc}{integer

The number of elements in the constant objective vector \eqn{{c}}{c}.


}
\item{ncolh}{integer

\eqn{{n}_{{H}}}{n_H}, the number of leading nonzero columns of the Hessian 
matrix \eqn{{H}}{H}. For FP and LP problems, ncolh must be set to zero.


}
\item{iobj}{integer

If \eqn{iobj > {0}}{iobj > 0}, row iobj of \eqn{{A}}{A} is a free row containing 
the nonzero elements of the vector \eqn{{c}}{c} appearing in the linear 
objective term \eqn{{c}^{T}{x}}{c^Tx}.


}
\item{objadd}{double

The constant \eqn{{q}}{q}, to be added to the objective for printing purposes. 
Typically \eqn{objadd = {0.0E0}}{objadd = 0.0E0}.


}
\item{prob}{string

The name for the problem. It is used in the printed solution and in some 
functions that output Basis files. A blank name may be used.


}
\item{acol}{double array

The nonzero elements of \eqn{{A}}{A}, ordered by increasing column index. 
Note that all elements must be assigned a value in the calling program.


}
\item{inda}{integer array

\eqn{inda[{i}]}{inda[i]} must contain the row index of the nonzero element 
stored in \eqn{acol[{i}]}{acol[i]} for \eqn{{i}={1} \ldots ne}{i=1  . . .  ne}. 
Thus a pair of values \eqn{\left(acol[{i}]inda[{i}]\right)}{(acol[i]inda[i])} 
contains a matrix element and its corresponding row index.


}
\item{loca}{integer array

\eqn{loca[{j}]}{loca[j]} must contain the index in acol and inda of the 
start of the \eqn{{j}}{j}th column for \eqn{{j}={1} \ldots n}{j=1  . . .  n}. 
Thus for \eqn{{j} = {1} : n}{j = 1 : n}, the entries of column \eqn{{j}}{j} 
are held in \eqn{acol[{k}:{l}]}{acol[k:l]} and their corresponding row 
indices are in \eqn{inda[{k}:{l}]}{inda[k:l]}, where \eqn{{k} = loca[{j}]}{k = loca[j]} 
and \eqn{{l} = loca[{j}+{1}] - {1}}{l = loca[j+1] - 1}. To specify the 
\eqn{{j}}{j}th column as empty, set \eqn{loca[{j}] = loca[{j}+{1}]}{loca[j] = loca[j+1]}. 
Note that the first and last elements of loca must be \eqn{loca[{1}] = 
{1}}{loca[1] = 1} and \eqn{loca[n+{1}] = ne + {1}}{loca[n+1] = ne + 1}. 
If your problem has no constraints, or just bounds on the variables, you 
may include a dummy \sQuote{free} row with a single (zero) element by setting 
\eqn{ne = {1}}{ne = 1}, \eqn{acol[{1}] = {0.0}}{acol[1] = 0.0}, \eqn{inda[{1}] 
= {1}}{inda[1] = 1}, \eqn{loca[{1}] = {1}}{loca[1] = 1}, and \eqn{loca[{j}] 
= {2}}{loca[j] = 2}, for \eqn{{j} = {2} : n + {1}}{j = 2 : n + 1}. This 
row is made \sQuote{free} by setting its bounds to be \eqn{bl[n+{1}] = 
- {bigbnd}}{bl[n+1] = - bigbnd} and \eqn{bu[n+{1}] = {bigbnd}}{bu[n+1] = bigbnd}, 
where \eqn{{bigbnd}}{bigbnd} is the value of the optional argument infiniteboundsize.


}
\item{bl}{double array

\eqn{{l}}{l}, the lower bounds for all the variables and general constraints, 
in the following order. The first n elements of bl must contain the bounds 
on the variables \eqn{{x}}{x}, and the next m elements the bounds for the 
general linear constraints \eqn{{A}{x}}{Ax} (which, equivalently, are the 
bounds for the slacks, \eqn{{s}}{s}) and the free row (if any). To fix 
the \eqn{{j}}{j}th variable, set \eqn{bl[{j}] = bu[{j}] = {\beta}}{bl[j] = bu[j] = \beta}, 
say, where \eqn{\mathrm{abs}({\beta}) < {bigbnd}}{abs(\beta) < bigbnd}. To specify 
a nonexistent lower bound (i.e., \eqn{{l}_{{j}} = { - {\infty}}}{l_j =  -  infinity}), 
set \eqn{bl[{j}] \leq - {bigbnd}}{bl[j]  <=  - bigbnd}. Here, \eqn{{bigbnd}}{bigbnd} 
is the value of the optional argument infiniteboundsize. To specify the 
\eqn{{j}}{j}th constraint as an \emph{equality}, set \eqn{bl[n+{j}] = bu[n+{j}] 
= {\beta}}{bl[n+j] = bu[n+j] = \beta}, say, where \eqn{\mathrm{abs}({\beta}) < {bigbnd}}{abs(\beta) < bigbnd}. 
Note that the lower bound corresponding to the free row must be set to 
\eqn{{ - {\infty}}}{ -  infinity} and stored in \eqn{bl[n+iobj]}{bl[n+iobj]}.


}
\item{bu}{double array

\eqn{{u}}{u}, the upper bounds for all the variables and general constraints, 
in the following order. The first n elements of bu must contain the bounds 
on the variables \eqn{{x}}{x}, and the next m elements the bounds for the 
general linear constraints \eqn{{A}{x}}{Ax} (which, equivalently, are the 
bounds for the slacks, \eqn{{s}}{s}) and the free row (if any). To specify 
a nonexistent upper bound (i.e., \eqn{{u}_{{j}} = { + {\infty}}}{u_j =  +  infinity}), 
set \eqn{bu[{j}] \geq {bigbnd}}{bu[j]  >=  bigbnd}. Note that the upper bound 
corresponding to the free row must be set to \eqn{{ + {\infty}}}{ +  infinity} 
and stored in \eqn{bu[n+iobj]}{bu[n+iobj]}.


}
\item{c}{double array

Contains the explicit objective vector \eqn{{c}}{c} (if any). If the problem 
is of type FP, or if \eqn{lenc = {0}}{lenc = 0}, then c is not referenced. 
(In that case, c may be dimensioned eqn1, or it could be any convenient 
array.)


}{double array

Contains the explicit objective vector \eqn{{c}}{c} (if any). If the problem 
is of type FP, or if \eqn{lenc = {0}}{lenc = 0}, then c is not referenced. 
(In that case, c may be dimensioned eqn1, or it could be any convenient 
array.)


}
\item{names}{string array

The optional column and row names, respectively.


}
\item{helast}{integer array

Defines which variables are to be treated as being elastic in elastic mode. 
The allowed values of helast are: helast need not be assigned if optional 
argument \eqn{elasticmode={0}}{elasticmode=0}.


}
\item{hs}{integer array

If \eqn{start=\mathtt{'C'}, \mathtt{'B'}}{start='C', 'B'}, and a Basis 
file of some sort is to be input (see the description of the optional arguments 
oldbasisfile, insertfile or loadfile), then hs and x need not be set 
at all.


}
\item{x}{double array

The initial values of the variables \eqn{{x}}{x}, and, if \eqn{start=\mathtt{'W'}}{start='W'}, 
the slacks \eqn{{s}}{s}, i.e., \eqn{\left({x}{s}\right)}{(xs)}. (See the 
description for argument hs.)


}
\item{ns}{integer

\eqn{{n}_{{S}}}{n_S}, the number of superbasics. For QP problems, ns need 
not be specified if \eqn{start=\mathtt{'C'}}{start='C'}, but must retain 
its value from a previous call when \eqn{start=\mathtt{'W'}}{start='W'}. 
For FP and LP problems, ns need not be initialized.


}
\item{optlist}{options list

Optional parameters may be listed, as shown in the following table:

\tabular{lll}{
\bold{Name} \tab \bold{Type} \tab \bold{Default} \cr
\code{Check Frequency} \tab \emph{integer} \tab Default \eqn{\mbox{} = {60}}{ = 60} \cr
\code{Crash Option} \tab \emph{integer} \tab Default \eqn{\mbox{} = {3}}{ = 3} \cr
\code{Crash Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = {0.1}}{ = 0.1} \cr
\code{Defaults} \tab \emph{} \tab  \cr
\code{Dump File} \tab \emph{integer} \tab Default \eqn{\mbox{} = {0}}{ = 0} \cr
\code{Load File} \tab \emph{integer} \tab Default \eqn{\mbox{} = {0}}{ = 0} \cr
\code{Elastic Mode} \tab \emph{integer} \tab Default \eqn{\mbox{} = {1}}{ = 1} \cr
\code{Elastic Objective} \tab \emph{integer} \tab Default \eqn{\mbox{} = {1}}{ = 1} \cr
\code{Elastic Weight} \tab \emph{double} \tab Default \eqn{\mbox{} = {1.0}}{ = 1.0} \cr
\code{Expand Frequency} \tab \emph{integer} \tab Default \eqn{\mbox{} = {10000}}{ = 10000} \cr
\code{Factorization Frequency} \tab \emph{integer} \tab Default \eqn{\mbox{} = {100}\left({LP}\right)}{ = 100(LP)} or \eqn{{50}\left({QP}\right)}{50(QP)} \cr
\code{Feasibility Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = \mathrm{max}\left\{{10}^{{ - {6}}}\sqrt{{\epsilon}}\right\}}{ = max10^ - 6sqrt(\epsilon)} \cr
\code{Infinite Bound Size} \tab \emph{double} \tab Default \eqn{\mbox{} = {10}^{{20}}}{ = 10^20} \cr
\code{Iterations Limit} \tab \emph{integer} \tab Default \eqn{\mbox{} = \mathrm{max}\left\{{10000}{{10}\mathrm{max}\left\{{m}{n}\right\}}\right\}}{ = max1000010maxmn} \cr
\code{LU Density Tolerance} \tab \emph{double} \tab Default 
 \eqn{\mbox{} = {0.6}}{ = 0.6}
 \cr
\code{LU Singularity Tolerance} \tab \emph{double} \tab Default 
 \eqn{\mbox{} = {\epsilon}^{\frac{{2}}{{3}}}}{ = \epsilon^(2)/(3)}
 \cr
\code{LU Factor Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = {100.0}}{ = 100.0} \cr
\code{LU Update Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = {10.0}}{ = 10.0} \cr
\code{LU Partial Pivoting} \tab \emph{} \tab Default \cr
\code{LU Complete Pivoting} \tab \emph{} \tab  \cr
\code{LU Rook Pivoting} \tab \emph{} \tab  \cr
\code{Minimize} \tab \emph{} \tab Default \cr
\code{Maximize} \tab \emph{} \tab  \cr
\code{Feasible Point} \tab \emph{} \tab  \cr
\code{New Basis File} \tab \emph{integer} \tab Default \eqn{\mbox{} = {0}}{ = 0} \cr
\code{Backup Basis File} \tab \emph{integer} \tab Default \eqn{\mbox{} = {0}}{ = 0} \cr
\code{Save Frequency} \tab \emph{integer} \tab Default \eqn{\mbox{} = {100}}{ = 100} \cr
\code{Nolist} \tab \emph{} \tab Default \cr
\code{List} \tab \emph{} \tab  \cr
\code{Old Basis File} \tab \emph{integer} \tab Default \eqn{\mbox{} = {0}}{ = 0} \cr
\code{Optimality Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = \mathrm{max}\left\{{10}^{{ - {6}}}\sqrt{{\epsilon}}\right\}}{ = max10^ - 6sqrt(\epsilon)} \cr
\code{Partial Price} \tab \emph{integer} \tab Default \eqn{\mbox{} = {10}\left({LP}\right)}{ = 10(LP)} or \eqn{{1}\left({QP}\right)}{1(QP)} \cr
\code{Pivot Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = {\epsilon}^{\frac{{2}}{{3}}}}{ = \epsilon^(2)/(3)} \cr
\code{Print File} \tab \emph{integer} \tab Default \eqn{\mbox{} = {0}}{ = 0} \cr
\code{Print Frequency} \tab \emph{integer} \tab Default \eqn{\mbox{} = {100}}{ = 100} \cr
\code{Print Level} \tab \emph{integer} \tab Default \eqn{\mbox{} = {1}}{ = 1} \cr
\code{Punch File} \tab \emph{integer} \tab Default \eqn{= {0}}{= 0} \cr
\code{Insert File} \tab \emph{integer} \tab Default \eqn{= {0}}{= 0} \cr
\code{QPSolver Cholesky} \tab \emph{} \tab Default \cr
\code{QPSolver CG} \tab \emph{} \tab  \cr
\code{QPSolver QN} \tab \emph{} \tab  \cr
\code{Reduced Hessian Dimension} \tab \emph{integer} \tab Default \eqn{= {1}\left({LP}\right)\mbox{ or }\mathrm{min}\left({2000}{{n}_{{H}} + {1}}{n}\right)\left({QP}\right)}{= 1(LP) or min(2000n_H + 1n)(QP)} \cr
\code{Scale Option} \tab \emph{integer} \tab Default \eqn{\mbox{} = {2}}{ = 2} \cr
\code{Scale Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = {0.9}}{ = 0.9} \cr
\code{Scale Print} \tab \emph{} \tab  \cr
\code{Solution File} \tab \emph{integer} \tab Default \eqn{\mbox{} = {0}}{ = 0} \cr
\code{Summary File} \tab \emph{integer} \tab Default \eqn{\mbox{} = {0}}{ = 0} \cr
\code{Summary Frequency} \tab \emph{integer} \tab Default \eqn{\mbox{} = {100}}{ = 100} \cr
\code{Superbasics Limit} \tab \emph{integer} \tab Default \eqn{\mbox{} = {1}\left({LP}\right)\mbox{ or }\mathrm{min}\left\{{{n}_{{H}} + {1}}{n}\right\}\left({QP}\right)}{ = 1(LP) or minn_H + 1n(QP)} \cr
\code{Suppress Parameters} \tab \emph{} \tab  \cr
\code{System Information No} \tab \emph{} \tab Default \cr
\code{System Information Yes} \tab \emph{} \tab  \cr
\code{Timing Level} \tab \emph{integer} \tab Default \eqn{\mbox{} = {0}}{ = 0} \cr
\code{Unbounded Step Size} \tab \emph{double} \tab Default \eqn{\mbox{} = {infbnd}}{ = infbnd} \cr
}
}
\item{ne}{integer: \strong{default} = nrow(acol)

The number of nonzero elements in \eqn{{A}}{A}.


}
\item{nname}{integer: \strong{default} = nrow(names)

The number of column (i.e., variable) and row names supplied in the array 
names. 

\eqn{nname = {1}}{nname = 1}: There are no names. Default names will be 
used in the printed output.

\eqn{nname = n + m}{nname = n + m}: All names must be supplied.




}
}

\details{
R interface to the NAG Fortran routine E04NQF.}

\value{

\item{HS}{integer array

The final states of the variables and slacks \eqn{\left({x}{s}\right)}{(xs)}. 
The significance of each possible value of \eqn{hs[{j}]}{hs[j]} is as follows:


}
\item{X}{double array

The final values of the variables and slacks \eqn{\left({x}{s}\right)}{(xs)}.


}
\item{PI}{double array

Contains the dual variables \eqn{{\pi}}{\pi} (a set of Lagrange multipliers 
(shadow prices) for the general constraints).


}
\item{RC}{double array

Contains the reduced costs, \eqn{{g} - \left(\begin{array}{cc} {A}&-{I}\\ 
\end{array}\right) ^{T}{\pi}}{g - 
(
A -I
)
^T\pi}. The vector \eqn{{g}}{g} is the gradient of the objective if x is 
feasible, otherwise it is the gradient of the Phase 1 objective. In the 
former case, \eqn{{g}\left({i}\right) = {0}}{g(i) = 0}, for \eqn{{i} = 
n + {1} : m}{i = n + 1 : m}, hence \eqn{rc\left(n + {1} : m\right) = {\pi}}{rc(n + 1 : m) = \pi}.


}
\item{NS}{integer

The final number of superbasics. This will be zero for FP and LP problems.


}
\item{NINF}{integer

The number of infeasibilities.


}
\item{SINF}{double

The sum of the scaled infeasibilities. This will be zero if \eqn{ninf = 
{0}}{ninf = 0}, and is most meaningful when \eqn{scaleoption={0}}{scaleoption=0}.


}
\item{OBJ}{double

The value of the objective function.


}
\item{IFAIL}{integer

\eqn{\mathrm{ifail} =0}{ifail =0} unless the function detects an error 
or a warning has been flagged (see the Errors section in Fortran library 
documentation).


}
}

\references{
\url{http://www.nag.co.uk/numeric/FL/nagdoc_fl23/pdf/E04/e04nqf.pdf}
}

\author{NAG}

\examples{

optlist<-list()

ifail<-0
qphx=function(ncolh,x,nstate){

hx<-as.matrix(mat.or.vec(ncolh,1))
hx[1]<-2\%*\%x[1]
hx[2]<-2\%*\%x[2]
hx[3]<-2\%*\%(x[3]+x[4])
hx[4]<-hx[3]
hx[5]<-2\%*\%x[5]
hx[6]<-2\%*\%(x[6]+x[7])
hx[7]<-hx[6]
list(HX=as.matrix(hx))
}

start<-'C'

m<-8

n<-7

lenc<-0

ncolh<-7

iobj<-8

objadd<-0

prob<-''

acol<-matrix(c(0.02,0.02,0.03,1,0.7,0.02,0.15,-200,0.06,0.75,0.03,0.04,0.05,0.04,1,-2000,0.02,1,0.01,0.08,0.08,0.8,-2000,1,0.12,0.02,0.02,0.75,0.04,-2000,0.01,0.8,0.02,1,0.02,0.06,0.02,-2000,1,0.01,0.01,0.97,0.01,400,0.97,0.03,1,400),nrow=48,ncol=1,byrow=TRUE)



inda<-matrix(c(7,5,3,1,6,4,2,8,7,6,5,4,3,2,1,8,2,1,4,3,7,6,8,1,7,3,4,6,2,8,5,6,7,1,2,3,4,8,1,2,3,6,7,8,7,2,1,8),nrow=48,ncol=1,byrow=TRUE)



loca<-matrix(c(1,9,17,24,31,39,45,49),nrow=8,ncol=1,byrow=TRUE)



bl<-matrix(c(0,0,400,100,0,0,0,2000,-9.999999999999999e+24,-9.999999999999999e+24,-9.999999999999999e+24,-9.999999999999999e+24,1500,250,-9.999999999999999e+24),nrow=15,ncol=1,byrow=TRUE)



bu<-matrix(c(200,2500,800,700,1500,9.999999999999999e+24,9.999999999999999e+24,2000,60,100,40,30,9.999999999999999e+24,300,9.999999999999999e+24),nrow=15,ncol=1,byrow=TRUE)



c<-matrix(c(0),nrow=1,ncol=1,byrow=TRUE)



names<-matrix(c('...X1...','...X2...','...X3...','...X4...','...X5...','...X6...','...X7...','..ROW1..','..ROW2..','..ROW3..','..ROW4..','..ROW5..','..ROW6..','..ROW7..','..COST..'),nrow=15,byrow=TRUE)



helast<-matrix(c(0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),nrow=15,ncol=1,byrow=TRUE)



hs<-matrix(c(0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),nrow=15,ncol=1,byrow=TRUE)



x<-matrix(c(0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),nrow=15,ncol=1,byrow=TRUE)



ns<-0

e04nq(start,qphx,m,n,lenc,ncolh,iobj,objadd,prob,acol,inda,loca,bl,bu,c,names,helast,hs,x,ns,optlist)

}

\keyword{math}

\keyword{optimize}
\concept{LP}
\concept{QP}
