
\name{e04vj}
\alias{e04vj}

\title{e04vj: Determine the pattern of nonzeros in the Jacobian matrix for e04vh}

\description{
e04vj may be used before e04vh to determine the sparsity pattern for the 
Jacobian.

}

\usage{
e04vj(nf, usrfun, lena, leng, x, xlow, xupp,
      n = nrow(x))
}

\arguments{

\item{nf}{integer

\eqn{{nf}}{nf}, the number of problem functions in \eqn{{F}\left({x}\right)}{F(x)}, 
including the objective function (if any) and the linear and nonlinear 
constraints. Simple upper and lower bounds on \eqn{{x}}{x} can be defined 
using the arguments xlow and xupp and should not be included in \eqn{{F}}{F}.


}
\item{usrfun}{function

usrfun must define the problem functions \eqn{{F}\left({x}\right)}{F(x)}.  This function is passed to e04vj as the external argument usrfun.

\code{(STATUS,F,G) = usrfun(status,n,x,needf,nf,f,needg,leng,g)
}


}
\item{lena}{integer

Lena should be an \emph{overestimate} of the number of elements in the 
linear part of the Jacobian.


}
\item{leng}{integer

Leng should be an \emph{overestimate} of the number of elements in the 
nonlinear part of the Jacobian.


}
\item{x}{double array

An initial estimate of the variables \eqn{{x}}{x}. The contents of \eqn{{x}}{x} 
will be used by e04vj in the call of usrfun, and so each element of x 
should be within the bounds given by xlow xupp.


}
\item{xlow}{double array}
\item{xupp}{double array

Contain the lower and upper bounds \eqn{{l}_{{x}}}{l_x} and \eqn{{u}_{{x}}}{u_x} 
on the variables \eqn{{x}}{x}.


}
\item{n}{integer: \strong{default} = nrow(x)

\eqn{{n}}{n}, the number of variables.


}
}

\details{
R interface to the NAG Fortran routine E04VJF.}

\value{

\item{IAFUN}{integer array}
\item{JAVAR}{integer array


}
\item{NEA}{integer

Is the number of nonzero entries in \eqn{{A}}{A} such that \eqn{{F}\left({x}\right) 
= {f}\left({x}\right) + {A}{x}}{F(x) = f(x) + Ax}.


}
\item{A}{double array

Define the coordinates \eqn{\left({i}{j}\right)}{(ij)} and values \eqn{{A}_{{{i}{j}}}}{A_ij} 
of the nonzero elements of the linear part \eqn{{A}}{A} of the function 
\eqn{{F}\left({x}\right) = {f}\left({x}\right) + {A}{x}}{F(x) = f(x) + Ax}.


}
\item{IGFUN}{integer array}
\item{JGVAR}{integer array

Define the coordinates \eqn{\left({i}{j}\right)}{(ij)} of the nonzero elements 
of \eqn{{G}}{G}, the nonlinear part of the derivatives \eqn{{J}\left({x}\right) 
= {G}\left({x}\right) + {A}}{J(x) = G(x) + A} of the function \eqn{{F}\left({x}\right) 
= {f}\left({x}\right) + {A}{x}}{F(x) = f(x) + Ax}.


}
\item{NEG}{integer

The number of nonzero entries in \eqn{{G}}{G}.


}
\item{IFAIL}{integer

\eqn{\mathrm{ifail} =0}{ifail =0} unless the function detects an error 
or a warning has been flagged (see the Errors section in Fortran library 
documentation).


}
}

\references{
\url{http://www.nag.co.uk/numeric/FL/nagdoc_fl23/pdf/E04/e04vjf.pdf}
}

\author{NAG}

\examples{

optlist <- list()

ifail <- 0
usrfun = function(status, n, x, needf, nf, f, needg, 
    leng, g) {
    
    f[1] <- 1000 \%*\% sin(-x[1] - 0.25) + 1000 \%*\% sin(-x[2] - 
        0.25) - x[3]
    f[2] <- 1000 \%*\% sin(x[1] - 0.25) + 1000 \%*\% sin(x[1] - x[2] - 
        0.25) - x[4]
    f[3] <- 1000 \%*\% sin(x[2] - x[1] - 0.25) + 1000 \%*\% sin(x[2] - 
        0.25)
    f[4] <- -x[1] + x[2]
    f[5] <- x[1] - x[2]
    f[6] <- 1e-06 \%*\% x[3]^3 + 2e-06 \%*\% x[4]^3/3 + 3 \%*\% x[3] + 
        2 \%*\% x[4]
    list(STATUS = as.integer(status), F = as.matrix(f), G = as.matrix(g))
}

nf <- 6

lena <- 300

leng <- 300

x <- matrix(c(0, 0, 0, 0), nrow = 4, ncol = 1, byrow = TRUE)



xlow <- matrix(c(-0.55, -0.55, 0, 0), nrow = 4, ncol = 1, 
    byrow = TRUE)



xupp <- matrix(c(0.55, 0.55, 1200, 1200), nrow = 4, 
    ncol = 1, byrow = TRUE)



e04vj(nf, usrfun, lena, leng, x, xlow, xupp) 

}

\keyword{math}

\keyword{optimize}
\concept{sparsity}
\concept{pattern}
