
\name{e04wd}
\alias{e04wd}

\title{e04wd: Solves the nonlinear programming (NP) problem}

\description{
e04wd is designed to minimize an arbitrary smooth function subject to 
constraints (which may include simple bounds on the variables, linear constraints 
and smooth nonlinear constraints) using a sequential quadratic programming 
(SQP) method. As many first derivatives as possible should be supplied 
by you; any unspecified derivatives are approximated by finite differences. 
It is not intended for large sparse problems.

e04wd may also be used for unconstrained, bound-constrained and linearly 
constrained optimization.

e04wd uses \bold{forward} \bold{communication} for evaluating the objective 
function, the nonlinear constraint functions, and any of their derivatives.

The initialization function e04wc \bold{must} have been called before to 
calling e04wd.

}

\usage{
e04wd(a, bl, bu, confun, objfun, istate, ccon, cjac, clamda, h, x, optlist,
      n = nrow(x),
      nclin = nrow(a),
      ncnln = nrow(cjac))
}

\arguments{

\item{a}{double array

The \eqn{{i}}{i}th row of a contains the \eqn{{i}}{i}th row of the matrix 
\eqn{{A}_{{L}}}{A_L} of general linear constraints in eqn1. That is, the 
\eqn{{i}}{i}th row contains the coefficients of the \eqn{{i}}{i}th general 
linear constraint for \eqn{{i}={1} \ldots nclin}{i=1  . . .  nclin}.


}
\item{bl}{double array}
\item{bu}{double array

Bl must contain the lower bounds and bu the upper bounds for all the constraints, 
in the following order. The first \eqn{{n}}{n} elements of each array must 
contain the bounds on the variables, the next \eqn{{n}_{{L}}}{n_L} elements 
the bounds for the general linear constraints (if any) and the next \eqn{{n}_{{N}}}{n_N} 
elements the bounds for the general nonlinear constraints (if any). To 
specify a nonexistent lower bound (i.e., \eqn{{l}_{{j}} = { - {\infty}}}{l_j =  -  infinity}), 
set \eqn{bl[{j}] \leq { - {bigbnd}}}{bl[j]  <=   - bigbnd}, and to specify 
a nonexistent upper bound (i.e., \eqn{{u}_{{j}} = { + {\infty}}}{u_j =  +  infinity}), 
set \eqn{bu[{j}] \geq {bigbnd}}{bu[j]  >=  bigbnd}; where \eqn{{bigbnd}}{bigbnd} 
is the optional argument infiniteboundsize. To specify the \eqn{{j}}{j}th 
constraint as an \emph{equality}, set \eqn{bl[{j}] = bu[{j}] = {\beta}}{bl[j] = bu[j] = \beta}, 
say, where \eqn{\mathrm{abs}({\beta}) < {bigbnd}}{abs(\beta) < bigbnd}.


}
\item{confun}{function

confun must calculate the vector \eqn{{c}\left({x}\right)}{c(x)} of nonlinear constraint functions and (optionally) its Jacobian, \eqn{\frac{{ \partial {c}}}{{ \partial {x}}}}{( \partial c)/( \partial x)}, for a specified \eqn{{n}}{n}-vector \eqn{{x}}{x}.  If there are no nonlinear constraints (i.e., \eqn{ncnln = {0}}{ncnln = 0}), e04wd will never call confun, so it may be the dummy function e04wdp.  (e04wdp is included in the NAG Library).
If there are nonlinear constraints, the first call to confun will occur before the first call to objfun.

\code{(MODE,CCON,CJAC) = confun(mode,ncnln,n,needc,x,cjac,nstate)
}


}
\item{objfun}{function

objfun must calculate the objective function \eqn{{F}\left({x}\right)}{F(x)} and (optionally) its gradient \eqn{{g}\left({x}\right) = \frac{{ \partial {F}}}{{ \partial {x}}}}{g(x) = ( \partial F)/( \partial x)} for a specified \eqn{{n}}{n}-vector \eqn{{x}}{x}.

\code{(MODE,OBJF,GRAD) = objfun(mode,n,x,grad,nstate)
}


}
\item{istate}{integer array

Is an integer array that need not be initialized if e04wd is called with 
the coldstart option (the default).


}
\item{ccon}{double array

Ccon need not be initialized if the (default) optional argument coldstart 
is used.


}
\item{cjac}{double array

In general, cjac need not be initialized before the call to e04wd. However, 
if \eqn{derivativelevel={2}, {3}}{derivativelevel=2, 3}, any constant 
elements of cjac may be initialized. Such elements need not be reassigned 
on subsequent calls to confun.


}
\item{clamda}{double array

Need not be set if the (default) optional argument coldstart is used.


}
\item{h}{double array

H need not be initialized if the (default) optional argument coldstart 
is used, and will be set to the identity.


}
\item{x}{double array

An initial estimate of the solution.


}
\item{optlist}{options list

Optional parameters may be listed, as shown in the following table:

\tabular{lll}{
\bold{Name} \tab \bold{Type} \tab \bold{Default} \cr
\code{Central Difference Interval} \tab \emph{double} \tab Default \eqn{\mbox{} = {\epsilon}_{{r}}^{\frac{{1}}{{3}}}}{ = \epsilon_r^(1)/(3)} \cr
\code{Check Frequency} \tab \emph{integer} \tab Default \eqn{\mbox{} = {60}}{ = 60} \cr
\code{Cold Start} \tab \emph{} \tab Default \cr
\code{Warm Start} \tab \emph{} \tab  \cr
\code{Crash Option} \tab \emph{integer} \tab Default \eqn{\mbox{} = {3}}{ = 3} \cr
\code{Crash Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = {0.1}}{ = 0.1} \cr
\code{Defaults} \tab \emph{} \tab  \cr
\code{Derivative Level} \tab \emph{integer} \tab Default \eqn{\mbox{} = {3}}{ = 3} \cr
\code{Derivative Linesearch} \tab \emph{} \tab Default \cr
\code{Nonderivative Linesearch} \tab \emph{} \tab  \cr
\code{Difference Interval} \tab \emph{double} \tab Default \eqn{\mbox{} = \sqrt{{\epsilon}_{{r}}}}{ = sqrt(\epsilon_r)} \cr
\code{Dump File} \tab \emph{integer} \tab Default \eqn{\mbox{} = {0}}{ = 0} \cr
\code{Load File} \tab \emph{integer} \tab Default \eqn{\mbox{} = {0}}{ = 0} \cr
\code{Elastic Weight} \tab \emph{double} \tab Default \eqn{\mbox{} = {10}^{{4}}}{ = 10^4} \cr
\code{Expand Frequency} \tab \emph{integer} \tab Default \eqn{\mbox{} = {10000}}{ = 10000} \cr
\code{Factorization Frequency} \tab \emph{integer} \tab Default \eqn{\mbox{} = {50}}{ = 50} \cr
\code{Function Precision} \tab \emph{double} \tab Default \eqn{\mbox{} = {\epsilon}^{{0.8}}}{ = \epsilon^0.8} \cr
\code{Hessian Full Memory} \tab \emph{} \tab Default if \eqn{{n} \leq {75}}{n  <=  75} \cr
\code{Hessian Limited Memory} \tab \emph{} \tab Default if \eqn{{n} > {75}}{n > 75} \cr
\code{Hessian Frequency} \tab \emph{integer} \tab Default \eqn{\mbox{} = {99999999}}{ = 99999999} \cr
\code{Hessian Updates} \tab \emph{integer} \tab Default \eqn{\mbox{} = hessianfrequency}{ = hessianfrequency} if hessianfullmemory, \eqn{{10}}{10} otherwise \cr
\code{Infinite Bound Size} \tab \emph{double} \tab Default \eqn{\mbox{} = {10}^{{20}}}{ = 10^20} \cr
\code{Iterations Limit} \tab \emph{integer} \tab Default \eqn{\mbox{} = \mathrm{max}\left({10000}{{10}\mathrm{max}\left({n}{{n}_{{L}} + {n}_{{N}}}\right)}\right)}{ = max(1000010max(nn_L + n_N))} \cr
\code{Linesearch Tolerance} \tab \emph{double} \tab Default \eqn{= {0.9}}{= 0.9} \cr
\code{Nolist} \tab \emph{} \tab Default \cr
\code{List} \tab \emph{} \tab  \cr
\code{LU Density Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = {0.6}}{ = 0.6} \cr
\code{LU Singularity Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = {\epsilon}^{\frac{{2}}{{3}}}}{ = \epsilon^(2)/(3)} \cr
\code{LU Factor Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = {1.10}}{ = 1.10} \cr
\code{LU Update Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = {1.10}}{ = 1.10} \cr
\code{LU Partial Pivoting} \tab \emph{} \tab Default \cr
\code{LU Complete Pivoting} \tab \emph{} \tab  \cr
\code{LU Rook Pivoting} \tab \emph{} \tab  \cr
\code{Major Feasibility Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = \mathrm{max}\left({10}^{{ - {6}}}\sqrt{{\epsilon}}\right)}{ = max(10^ - 6sqrt(\epsilon))} \cr
\code{Major Optimality Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = {2}\mathrm{max}\left({10}^{{ - {6}}}\sqrt{{\epsilon}}\right)}{ = 2max(10^ - 6sqrt(\epsilon))} \cr
\code{Major Iterations Limit} \tab \emph{integer} \tab Default \eqn{\mbox{} = \mathrm{max}\left({1000}{{3}\mathrm{max}\left({n}{{n}_{{L}} + {n}_{{N}}}\right)}\right)}{ = max(10003max(nn_L + n_N))} \cr
\code{Major Print Level} \tab \emph{integer} \tab Default \eqn{\mbox{} = {000001}}{ = 000001} \cr
\code{Major Step Limit} \tab \emph{double} \tab Default \eqn{\mbox{} = {2.0}}{ = 2.0} \cr
\code{Minimize} \tab \emph{} \tab Default \cr
\code{Maximize} \tab \emph{} \tab  \cr
\code{Feasible Point} \tab \emph{} \tab  \cr
\code{Minor Feasibility Tolerance} \tab \emph{} \tab  \cr
\code{Feasibility Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = \mathrm{max}\left\{{10}^{{ - {6}}}\sqrt{{\epsilon}}\right\}}{ = max10^ - 6sqrt(\epsilon)} \cr
\code{Minor Iterations Limit} \tab \emph{integer} \tab Default \eqn{\mbox{} = {500}}{ = 500} \cr
\code{Minor Print Level} \tab \emph{integer} \tab Default \eqn{\mbox{} = {1}}{ = 1} \cr
\code{New Basis File} \tab \emph{integer} \tab Default \eqn{\mbox{} = {0}}{ = 0} \cr
\code{Backup Basis File} \tab \emph{integer} \tab Default \eqn{\mbox{} = {0}}{ = 0} \cr
\code{Save Frequency} \tab \emph{integer} \tab Default \eqn{\mbox{} = {100}}{ = 100} \cr
\code{New Superbasics Limit} \tab \emph{integer} \tab Default \eqn{\mbox{} = {99}}{ = 99} \cr
\code{Old Basis File} \tab \emph{integer} \tab Default \eqn{\mbox{} = {0}}{ = 0} \cr
\code{Partial Price} \tab \emph{integer} \tab Default \eqn{\mbox{} = {1}}{ = 1} \cr
\code{Pivot Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = {\epsilon}^{\frac{{2}}{{3}}}}{ = \epsilon^(2)/(3)} \cr
\code{Print File} \tab \emph{integer} \tab Default \eqn{\mbox{} = {0}}{ = 0} \cr
\code{Print Frequency} \tab \emph{integer} \tab Default \eqn{\mbox{} = {100}}{ = 100} \cr
\code{Proximal Point Method} \tab \emph{integer} \tab Default \eqn{\mbox{} = {1}}{ = 1} \cr
\code{Punch File} \tab \emph{integer} \tab Default \eqn{= {0}}{= 0} \cr
\code{Insert File} \tab \emph{integer} \tab Default \eqn{= {0}}{= 0} \cr
\code{QPSolver Cholesky} \tab \emph{} \tab Default \cr
\code{QPSolver CG} \tab \emph{} \tab  \cr
\code{QPSolver QN} \tab \emph{} \tab  \cr
\code{Reduced Hessian Dimension} \tab \emph{integer} \tab Default \eqn{= \mathrm{min}\left({2000}{n}\right)}{= min(2000n)} \cr
\code{Scale Option} \tab \emph{integer} \tab Default \eqn{\mbox{} = {0}}{ = 0} \cr
\code{Scale Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = {0.9}}{ = 0.9} \cr
\code{Scale Print} \tab \emph{} \tab  \cr
\code{Solution File} \tab \emph{integer} \tab Default \eqn{\mbox{} = {0}}{ = 0} \cr
\code{Start Objective Check At Variable} \tab \emph{integer} \tab Default \eqn{\mbox{} = {1}}{ = 1} \cr
\code{Stop Objective Check At Variable} \tab \emph{integer} \tab Default \eqn{\mbox{} = {n}}{ = n} \cr
\code{Start Constraint Check At Variable} \tab \emph{integer} \tab Default \eqn{\mbox{} = {1}}{ = 1} \cr
\code{Stop Constraint Check At Variable} \tab \emph{integer} \tab Default \eqn{\mbox{} = {n}}{ = n} \cr
\code{Summary File} \tab \emph{integer} \tab Default \eqn{\mbox{} = {0}}{ = 0} \cr
\code{Summary Frequency} \tab \emph{integer} \tab Default \eqn{\mbox{} = {100}}{ = 100} \cr
\code{Superbasics Limit} \tab \emph{integer} \tab Default \eqn{\mbox{} = {n}}{ = n} \cr
\code{Suppress Parameters} \tab \emph{} \tab  \cr
\code{System Information No} \tab \emph{} \tab Default \cr
\code{System Information Yes} \tab \emph{} \tab  \cr
\code{Timing Level} \tab \emph{integer} \tab Default \eqn{\mbox{} = {0}}{ = 0} \cr
\code{Unbounded Objective} \tab \emph{double} \tab Default \eqn{\mbox{} = {1.0E+15}}{ = 1.0E+15} \cr
\code{Unbounded Step Size} \tab \emph{double} \tab Default \eqn{\mbox{} = {bigbnd}}{ = bigbnd} \cr
\code{Verify Level} \tab \emph{integer} \tab Default \eqn{\mbox{} = {0}}{ = 0} \cr
\code{Violation Limit} \tab \emph{double} \tab Default \eqn{\mbox{} = {1.0E+6}}{ = 1.0E+6} \cr
}
}
\item{n}{integer: \strong{default} = nrow(x)

\eqn{{n}}{n}, the number of variables.


}
\item{nclin}{integer: \strong{default} = nrow(a)

\eqn{{n}_{{L}}}{n_L}, the number of general linear constraints.


}
\item{ncnln}{integer: \strong{default} = nrow(cjac)

\eqn{{n}_{{N}}}{n_N}, the number of nonlinear constraints.


}
}

\details{
R interface to the NAG Fortran routine E04WDF.}

\value{

\item{MAJITS}{integer

The number of major iterations performed.


}
\item{ISTATE}{integer array

Describes the status of the constraints \eqn{{l} \leq {r}\left({x}\right) 
\leq {u}}{l  <=  r(x)  <=  u}. For the \eqn{{j}}{j}th lower or upper bound, 
\eqn{{j} = {1} , {2} , \ldots , n + nclin + ncnln}{j = 1 , 2 ,  . . .  , n + nclin + ncnln}, 
the possible values of \eqn{istate[{j}]}{istate[j]} are as follows (see 
the figure in the Fortran library documentation). \eqn{{\delta}}{\delta} is the appropriate 
feasibility tolerance.


}
\item{CCON}{double array

If \eqn{ncnln > {0}}{ncnln > 0}, \eqn{ccon[{i}]}{ccon[i]} contains the 
value of the \eqn{{i}}{i}th nonlinear constraint function \eqn{{c}_{{i}}}{c_i} 
at the final iterate for \eqn{{i}={1} \ldots ncnln}{i=1  . . .  ncnln}.


}
\item{CJAC}{double array

If \eqn{ncnln > {0}}{ncnln > 0}, cjac contains the Jacobian matrix of the 
nonlinear constraint functions at the final iterate, i.e., \eqn{cjac[{i}, 
{j}]}{cjac[i, j]} contains the partial derivative of the \eqn{{i}}{i}th 
constraint function with respect to the \eqn{{j}}{j}th variable for \eqn{{j}={1} 
\ldots n}{j=1  . . .  n} for \eqn{{i}={1} \ldots ncnln}{i=1  . . .  ncnln}. (See 
the discussion of argument cjac under confun.)


}
\item{CLAMDA}{double array

The values of the QP multipliers from the last QP subproblem. \eqn{clamda[{j}]}{clamda[j]} 
should be non-negative if \eqn{istate[{j}] = {1}}{istate[j] = 1} and non-positive 
if \eqn{istate[{j}] = {2}}{istate[j] = 2}.


}
\item{OBJF}{double

The value of the objective function at the final iterate.


}
\item{GRAD}{double array

The gradient of the objective function (or its finite difference approximation) 
at the final iterate.


}
\item{H}{double array

Contains the Hessian of the Lagrangian at the final estimate \eqn{{x}}{x}.


}
\item{X}{double array

The final estimate of the solution.


}
\item{IFAIL}{integer

\eqn{\mathrm{ifail} =0}{ifail =0} unless the function detects an error 
or a warning has been flagged (see the Errors section in Fortran library 
documentation).


}
}

\references{
\url{http://www.nag.co.uk/numeric/FL/nagdoc_fl23/pdf/E04/e04wdf.pdf}
}

\author{NAG}

\examples{

optlist <- list()

ifail <- 0
confun = function(mode, ncnln, n, needc, x, cjac, 
    nstate) {
    ldcj <- nrow(cjac)
    
    ccon <- as.matrix(mat.or.vec(ncnln, 1))
    
    if (nstate == 1) {
        
        cjac <- as.matrix(mat.or.vec(ncnln, n))
        
    }
    
    if (needc[1] > 0) {
        
        if (mode == 0 || mode == 2) {
            
            ccon[1] <- x[1]^2 + x[2]^2 + x[3]^2 + x[4]^2
            
        }
        if (mode == 1 || mode == 2) {
            
            cjac[1, 1] <- 2 \%*\% x[1]
            
            cjac[1, 2] <- 2 \%*\% x[2]
            
            cjac[1, 3] <- 2 \%*\% x[3]
            
            cjac[1, 4] <- 2 \%*\% x[4]
            
        }
    }
    
    if (needc[2] > 0) {
        
        if (mode == 0 || mode == 2) {
            
            ccon[2] <- x[1] \%*\% x[2] \%*\% x[3] \%*\% x[4]
            
        }
        if (mode == 1 || mode == 2) {
            
            cjac[2, 1] <- x[2] \%*\% x[3] \%*\% x[4]
            
            cjac[2, 2] <- x[1] \%*\% x[3] \%*\% x[4]
            
            cjac[2, 3] <- x[1] \%*\% x[2] \%*\% x[4]
            
            cjac[2, 4] <- x[1] \%*\% x[2] \%*\% x[3]
            
        }
    }
    list(MODE = as.integer(mode), CCON = as.matrix(ccon), CJAC = as.matrix(cjac))
}
objfun = function(mode, n, x, grad, nstate) {
    
    
    if (mode == 0 || mode == 2) {
        
        objf <- x[1] \%*\% x[4] \%*\% (x[1] + x[2] + x[3]) + x[3]
        
    }
    
    if (mode == 1 || mode == 2) {
        
        grad[1] <- x[4] \%*\% (2 \%*\% x[1] + x[2] + x[3])
        
        grad[2] <- x[1] \%*\% x[4]
        
        grad[3] <- x[1] \%*\% x[4] + 1
        
        grad[4] <- x[1] \%*\% (x[1] + x[2] + x[3])
        
    }
    list(MODE = as.integer(mode), OBJF = objf, GRAD = as.matrix(grad))
}

a <- matrix(c(1, 1, 1, 1), nrow = 1, ncol = 4, byrow = TRUE)



bl <- matrix(c(1, 1, 1, 1, -1e+25, -1e+25, 25), nrow = 7, 
    ncol = 1, byrow = TRUE)



bu <- matrix(c(5, 5, 5, 5, 20, 40, 1e+25), nrow = 7, 
    ncol = 1, byrow = TRUE)



istate <- as.matrix(mat.or.vec(7, 1))

ccon <- as.matrix(mat.or.vec(2, 1))

cjac <- as.matrix(mat.or.vec(2, 4))

clamda <- as.matrix(mat.or.vec(7, 1))

h <- as.matrix(mat.or.vec(4, 4))

x <- matrix(c(1, 5, 5, 1), nrow = 4, ncol = 1, byrow = TRUE)



e04wd(a, bl, bu, confun, objfun, istate, ccon, cjac, 
    clamda, h, x, optlist) 

}

\keyword{math}

\keyword{optimize}
\concept{constraint}
\concept{derivative}
\concept{minimum}
\concept{nonlinear}
\concept{QP}
\concept{sequential}
