
\name{e05jb}
\alias{e05jb}

\title{e05jb: Global optimization by multi-level coordinate search, simple bounds, using 
function values only}

\description{
e05jb is designed to find the global minimum or maximum of an arbitrary 
function, subject to simple bound-constraints using a multi-level coordinate 
search method. Derivatives are not required, but convergence is only guaranteed 
if the objective function is continuous in a neighbourhood of a global 
optimum. It is not intended for large problems.

The initialization function e05ja \bold{must} have been called before calling 
e05jb.

}

\usage{
e05jb(objfun, ibound, iinit, bl, bu, list, numpts, initpt, monit, optlist,
      n = nrow(bl),
      sdlist = ncol(list))
}

\arguments{

\item{objfun}{function

objfun must evaluate the objective function \eqn{{F}\left({x}\right)}{F(x)} for a specified \eqn{{n}}{n}-vector \eqn{{x}}{x}.

\code{(F,INFORM) = objfun(n,x,nstate)
}


}
\item{ibound}{integer

Indicates whether the facility for dealing with bounds of special forms 
is to be used. ibound must be set to one of the following values. 

\eqn{ibound={0}}{ibound=0}: You will supply \eqn{{\ell}}{\ell} and \eqn{{u}}{u} 
individually.

\eqn{ibound={1}}{ibound=1}: There are no bounds on \eqn{{x}}{x}.

\eqn{ibound={2}}{ibound=2}: There are semi-infinite bounds \eqn{{0} \leq {x}}{0  <=  x}.

\eqn{ibound={3}}{ibound=3}: 
There are constant bounds \eqn{{\ell} = {\ell}_{{1}}}{\ell = \ell_1} and \eqn{{u} = 
{u}_{{1}}}{u = u_1}.




}
\item{iinit}{integer

Selects which initialization method to use. 

\eqn{iinit={0}}{iinit=0}: Simple initialization (boundary and midpoint), 
with \eqn{numpts[{i}] = {3}}{numpts[i] = 3}, \eqn{initpt[{i}] = {2}}{initpt[i] = 2} 
and \eqn{list[{i}, {j}] = \left({bl[{i}]}{\left(bl[{i}] + bu[{i}]\right) 
/ {2}}{bu[{i}]}\right)}{list[i, j] = (bl[i](bl[i] + bu[i]) / 2bu[i])}, 
for \eqn{{i} = {1} , {2} , \ldots , n}{i = 1 , 2 ,  . . .  , n} and \eqn{{j} 
= {1} , {2} , {3}}{j = 1 , 2 , 3}.

\eqn{iinit={1}}{iinit=1}: Simple initialization (off-boundary and midpoint), 
with \eqn{numpts[{i}] = {3}}{numpts[i] = 3}, \eqn{initpt[{i}] = {2}}{initpt[i] = 2} 
and \eqn{list[{i}, {j}] = \left({\left({5}bl[{i}] + bu[{i}]\right) / {6}}{\left(bl[{i}] 
+ bu[{i}]\right) / {2}}{\left(bl[{i}] + {5}bu[{i}]\right) / {6}}\right)}{list[i, j] = ((5bl[i] + bu[i]) / 6(bl[i] + bu[i]) / 2(bl[i] + 5bu[i]) / 6)}, 
for \eqn{{i} = {1} , {2} , \ldots , n}{i = 1 , 2 ,  . . .  , n} and \eqn{{j} 
= {1} , {2} , {3}}{j = 1 , 2 , 3}.

\eqn{iinit={2}}{iinit=2}: Initialization using linesearches.

\eqn{iinit={3}}{iinit=3}: You are providing your own initialization list.

\eqn{iinit={4}}{iinit=4}: 
Generate a random initialization list.




}
\item{bl}{double array}
\item{bu}{double array

\eqn{bl}{bl} is \eqn{{\ell}}{\ell}, the array of lower bounds. \eqn{bu}{bu} is 
\eqn{{u}}{u}, the array of upper bounds.


}
\item{list}{double array

This argument need not be set on entry if you wish to use one of the preset 
initialization methods (\eqn{iinit\ne{3}}{iinit != 3}).


}
\item{numpts}{integer array

This argument need not be set on entry if you wish to use one of the preset 
initialization methods (\eqn{iinit\ne{3}}{iinit != 3}).


}
\item{initpt}{integer array

This argument need not be set on entry if you wish to use one of the preset 
initialization methods (\eqn{iinit\ne{3}}{iinit != 3}).


}
\item{monit}{function

monit may be used to monitor the optimization process. It is invoked upon every successful completion of the procedure in which a sub-box is considered for splitting. It will also be called just before e05jb exits if that splitting procedure was not successful.

\code{(INFORM) = monit(n,ncall,xbest,icount,ninit,list,numpts,initpt,nbaskt,xbaskt,boxl,boxu,nstate)
}


}
\item{optlist}{options list

Optional parameters may be listed, as shown in the following table:

\tabular{lll}{
\bold{Name} \tab \bold{Type} \tab \bold{Default} \cr
\code{Defaults} \tab \emph{} \tab  \cr
\code{Function Evaluations Limit} \tab \emph{integer} \tab Default \eqn{\mbox{} = {100}{n}_{{r}}^{{2}}}{ = 100n_r^2} \cr
\code{Infinite Bound Size} \tab \emph{double} \tab Default \eqn{\mbox{} = {r}_{{max}}^{\frac{{1}}{{4}}}}{ = r_max^(1)/(4)} \cr
\code{Local Searches} \tab \emph{string} \tab Default \eqn{\mbox{} = \sQuote{\mbox{ON}}}{ = sQuoteON} \cr
\code{Local Searches Limit} \tab \emph{integer} \tab Default \eqn{\mbox{} = {50}}{ = 50} \cr
\code{Local Searches Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = {2}{\epsilon}}{ = 2\epsilon} \cr
\code{Minimize} \tab \emph{} \tab Default \cr
\code{Maximize} \tab \emph{} \tab  \cr
\code{Nolist} \tab \emph{} \tab Default \cr
\code{List} \tab \emph{} \tab  \cr
\code{Repeatability} \tab \emph{string} \tab Default \eqn{\mbox{} = \sQuote{\mbox{OFF}}}{ = sQuoteOFF} \cr
\code{Splits Limit} \tab \emph{integer} \tab Default \eqn{\mbox{} = \left\lfloor{d}\left({n}_{{r}} + {2}\right) / {3}\right\rfloor}{ = \lfloord(n_r + 2) / 3\rfloor} \cr
\code{Static Limit} \tab \emph{integer} \tab Default \eqn{\mbox{} = {3}{n}_{{r}}}{ = 3n_r} \cr
\code{Target Objective Error} \tab \emph{double} \tab Default \eqn{\mbox{} = {\epsilon}^{\frac{{1}}{{4}}}}{ = \epsilon^(1)/(4)} \cr
\code{Target Objective Safeguard} \tab \emph{double} \tab Default \eqn{\mbox{} = {\epsilon}^{\frac{{1}}{{2}}}}{ = \epsilon^(1)/(2)} \cr
\code{Target Objective Value} \tab \emph{double} \tab  \cr
}
}
\item{n}{integer: \strong{default} = nrow(bl)

\eqn{{n}}{n}, the number of variables.


}
\item{sdlist}{integer: \strong{default} = ncol(list)

. sdlist is, at least, the maximum over \eqn{{i}}{i} of the number of points 
in coordinate \eqn{{i}}{i} at which to split according to the initialization 
list list; that is, \eqn{sdlist \geq {max}_{{i}}numpts[{i}]}{sdlist  >=  max_inumpts[i]}.


}
}

\details{
R interface to the NAG Fortran routine E05JBF.}

\value{

\item{BL}{double array}
\item{BU}{double array

Unless \eqn{\mathrm{ifail} =1}{ifail =1}, \eqn{\mathrm{ifail} =2}{ifail =2} 
on exit, bl and bu are the actual arrays of bounds used by e05jb.


}
\item{LIST}{double array

Unless \eqn{\mathrm{ifail} =1}{ifail =1}, \eqn{\mathrm{ifail} =2}{ifail =2}, 
\eqn{\mathrm{ifail} =-999}{ifail =-999} on exit, the actual initialization 
data used by e05jb. If you wish to monitor the contents of list you are 
advised to do so solely through monit, not through the output value here.


}
\item{NUMPTS}{integer array

Unless \eqn{\mathrm{ifail} =1}{ifail =1}, \eqn{\mathrm{ifail} =2}{ifail =2}, 
\eqn{\mathrm{ifail} =-999}{ifail =-999} on exit, the actual initialization 
data used by e05jb.


}
\item{INITPT}{integer array

Unless \eqn{\mathrm{ifail} =1}{ifail =1}, \eqn{\mathrm{ifail} =2}{ifail =2}, 
\eqn{\mathrm{ifail} =-999}{ifail =-999} on exit, the actual initialization 
data used by e05jb.


}
\item{X}{double array

If \eqn{\mathrm{ifail} =0}{ifail =0}, contains an estimate of the global 
optimum (see also the Accuracy section in the Fortran library documentation).


}
\item{OBJ}{double

If \eqn{\mathrm{ifail} =0}{ifail =0}, contains the function value at x.


}
\item{IFAIL}{integer

\eqn{\mathrm{ifail} =0}{ifail =0} unless the function detects an error 
or a warning has been flagged (see the Errors section in Fortran library 
documentation).


}
}

\references{
\url{http://www.nag.co.uk/numeric/FL/nagdoc_fl23/pdf/E05/e05jbf.pdf}
}

\author{NAG}

\examples{

optlist <- list()

ifail <- 0
peaks <- function(x1, x2) {
    
    f = 3 * (1 - x1)^2 * exp(-(x1^2) - (x2 + 1)^2) - 10 * (x1/5 - 
        x1^3 - x2^5) * exp(-x1^2 - x2^2) - 1/3 * exp(-(x1 + 1)^2 - 
        x2^2)
    
}

objective = function(n, x, nstate) {
   

print(x[1])
print(x[2])   
    if (n == 2) {
        
        inform <- 0
        
    }
    else {
        
        inform <- -1
    }
    
    if (inform >= 0) {
        
        if (nstate == 1) {
            
            writeLines(toString(cat(sprintf("\n", "\n"))))
            
            
            writeLines(toString(cat(sprintf("OBJFUN was just called for the first time", 
                "\n"))))
            
            
        }
        f <- peaks(x[1], x[2])
        
    }
    list(F = f, INFORM = as.integer(inform))
}
monitor = function(n, ncall, xbest, icount, ninit, 
    list, numpts, initpt, nbaskt, xbaskt, boxl, boxu, nstate) {
    
    inform <- 0
    
    if (nstate == 0 || nstate == 1) {
        
        writeLines(toString(cat(sprintf("\n", "\n"))))
        
        
        writeLines(toString(cat(sprintf("*** Begin monitoring information ***", 
            "\n"))))
        
        
        writeLines(toString(cat(sprintf("\n", "\n"))))
        
        
    }
    
    if (nstate <= 0) {
        
        writeLines(toString(cat(sprintf("Total sub-boxes = \%s", 
            toString(icount[1]), "\n"))))
        
        
        writeLines(toString(cat(sprintf("Total function evaluations = \%s", 
            toString(ncall), "\n"))))
        
        
        writeLines(toString(cat(sprintf("Total function evaluations used in local searches = \%s", 
            toString(icount[2]), "\n"))))
        
        
        writeLines(toString(cat(sprintf("Total points used in local search = \%s", 
            toString(icount[3]), "\n"))))
        
        
        writeLines(toString(cat(sprintf("Total sweeps through levels = \%s", 
            toString(icount[4]), "\n"))))
        
        
        writeLines(toString(cat(sprintf("Total splits by init. list = \%s", 
            toString(icount[5]), "\n"))))
        
        
        writeLines(toString(cat(sprintf("Lowest level with nonsplit boxes = \%s", 
            toString(icount[6]), "\n"))))
        
        
        writeLines(toString(cat(sprintf("Number of candidate minima in the \%s", 
            "shopping basket\%s", " = \%s", toString(nbaskt), 
            "\n"))))
        
        
        writeLines(toString(cat(sprintf("Shopping basket:", "\n"))))
        
       print(xbaskt) 
        
        
        writeLines(toString(cat(sprintf("\n", "\n"))))
        
        
        writeLines(toString(cat(sprintf("*** End monitoring information ***", 
            "\n"))))
        
        
        writeLines(toString(cat(sprintf("\n", "\n"))))
        
        
    }
    list(INFORM = as.integer(inform))
}

prob <- "peaks"

xres <- 100

yres <- 100

bl <- matrix(c(-3, -3), nrow = 2, ncol = 1, byrow = TRUE)



bu <- -bl

fglob <- -6.55

xglob <- matrix(c(0.23, -1.63), nrow = 2, ncol = 1, 
    byrow = TRUE)



n <- length(bl)

if (ifail == 0) {
    
    writeLines(toString(cat(sprintf("\n", "\n"))))
    
    
    writeLines(toString(cat(sprintf("Solve with no options or init.-list data", 
        "\n"))))
    
    
    ibound <- 0
    
    iinit <- 0
    
    list <- as.matrix(mat.or.vec(n, 3))
    
    numpts <- as.matrix(mat.or.vec(n, 1))
    
    initpt <- as.matrix(mat.or.vec(n, 1))
    
    ans <- e05jb(objective, ibound, iinit, bl, bu, list, numpts, 
        initpt, monitor, optlist)
    bl <- ans$BL
    bu <- ans$BU
    list <- ans$LIST
    numpts <- ans$NUMPTS
    initpt <- ans$INITPT
    x <- ans$X
    obj <- ans$OBJ
    ifail <- ans$IFAIL
    
    ifail <- ans$IFAIL
    
    writeLines(toString(cat(sprintf("e05jbno options exited with ifail = \%s", 
        toString(ifail), "\n"))))
    
    
    if (ifail == 0) {
        
        writeLines(toString(cat(sprintf("xbest:", "\n"))))
        
        
        xbest <- ans$XBEST
       
	print(xbest) 
        writeLines(toString(cat(sprintf("\n"))))
        
        
        obj <- ans$OBJ
        
        writeLines(toString(cat(sprintf("obj = \%s", toString(obj), 
            "\n"))))
        
        
    }
    writeLines(toString(cat(sprintf("\n", "\n"))))
    
    
    writeLines(toString(cat(sprintf("Solve with options and init.-list data", 
        "\n"))))
    
    
   infbnd <-1.1579+077 
    iinit <- 3
    
    list <- as.matrix(mat.or.vec(n, 3))
    
    list[, 1] <- bl
    
    list[, 3] <- bu
    
    list[, 2] <- matrix(c(-1, 0), nrow = 2, ncol = 1, byrow = TRUE)
    
    
    
    numpts <- 3 * matrix(1, n, 1)
    
    initpt <- 2 * matrix(1, n, 1)
    
    ans <- e05jb(objective, ibound, iinit, bl, bu, list, numpts, 
        initpt, monitor, optlist)
    
    ifail <- ans$IFAIL
    
    writeLines(toString(cat(sprintf("e05jboptions exited with ifail = \%s", 
        toString(ifail), "\n"))))
    
    
    if (ifail == 0) {
        
        writeLines(toString(cat(sprintf("xbest:", "\n"))))
        
        
        xbest <- ans$X
       
print(xbest) 
        writeLines(toString(cat(sprintf("\n"))))
        
        
        obj <- ans$OBJ
        
        writeLines(toString(cat(sprintf("obj = \%s", toString(obj), 
            "\n"))))
        
        
    }
} 

}

\keyword{math}

\keyword{optimize}
\concept{bound}
\concept{constrained}
\concept{global}
\concept{multi-level}
\concept{coordinate}
