
\name{s18dc}
\alias{s18dc}

\title{s18dc: Modified Bessel functions K_nu + a(z), real a  >=  0, complex z, nu = 0 , 1 , 2 ,  . . . }

\description{
s18dc returns a sequence of values for the modified Bessel functions \eqn{{K}_{{{\nu} 
+ {n}}}\left({z}\right)}{K_\nu + n(z)} for complex \eqn{{z}}{z}, non-negative 
\eqn{{\nu}}{\nu} and \eqn{{n} = {0} , {1} , \ldots , {N} - {1}}{n = 0 , 1 ,  . . .  , N - 1}, 
with an option for exponential scaling.

}

\usage{
s18dc(fnu, z, n, scal)
}

\arguments{

\item{fnu}{double

\eqn{{\nu}}{\nu}, the order of the first member of the sequence of functions.


}
\item{z}{complex

The argument \eqn{{z}}{z} of the functions.


}
\item{n}{integer

\eqn{{N}}{N}, the number of members required in the sequence \eqn{{K}_{{\nu}}\left({z}\right) 
, {K}_{{{\nu} + {1}}}\left({z}\right) , \ldots , {K}_{{{\nu} + {N} - {1}}}\left({z}\right)}{K_\nu(z) , K_\nu + 1(z) ,  . . .  , K_\nu + N - 1(z)}.


}
\item{scal}{string

The scaling option. 

\eqn{scal=\mathtt{'U'}}{scal='U'}: The results are returned unscaled.

\eqn{scal=\mathtt{'S'}}{scal='S'}: The results are returned scaled by the 
factor \eqn{{e}^{{z}}}{e^z}.




}
}

\details{
R interface to the NAG Fortran routine S18DCF.}

\value{

\item{CY}{complex array

The \eqn{{N}}{N} required function values: \eqn{cy[{i}]}{cy[i]} contains 
\eqn{{K}_{{{\nu} + {i} - {1}}}\left({z}\right)}{K_\nu + i - 1(z)} for \eqn{{i}={1} 
\ldots {N}}{i=1  . . .  N}.


}
\item{NZ}{integer

The number of components of cy that are set to zero due to underflow. If 
\eqn{nz > {0}}{nz > 0} and \eqn{\mathrm{Re}\left({z}\right) \geq {0.0}}{Re(z)  >=  0.0}, 
elements \eqn{cy[{1}] , cy[{2}] , \ldots , cy[nz]}{cy[1] , cy[2] ,  . . .  , cy[nz]} 
are set to zero. If \eqn{\mathrm{Re}\left({z}\right) < {0.0}}{Re(z) < 0.0}, 
nz simply states the number of underflows, and not which elements they 
are.


}
\item{IFAIL}{integer

\eqn{\mathrm{ifail} =0}{ifail =0} unless the function detects an error 
or a warning has been flagged (see the Errors section in Fortran library 
documentation).


}
}

\references{
\url{http://www.nag.co.uk/numeric/FL/nagdoc_fl23/pdf/S/s18dcf.pdf}
}

\author{NAG}

\examples{

ifail<-0

fnu<-0

z<-complex(1,0.3,0.4)

n<-2

scal<-'U'

s18dc(fnu,z,n,scal)

}

\keyword{math}
\concept{Bessel}
\concept{modified}
