
#include <R.h>
#include <Rinternals.h>
#include "nag_rwrapper_utils.h"
#include "nagmk23.h"
#ifdef __cplusplus
extern "C" {
#endif
extern void NAG_CALL e04fcf_LSQFUN(Integer *IFLAG, CONST Integer *M,
                                   CONST Integer *N, CONST double XC[],
                                   double FVEC[], Integer IW[],
                                   CONST Integer *LIW, double W[],
                                   CONST Integer *LW);
extern void NAG_CALL e04fcf_LSQMON(CONST Integer *M, CONST Integer *N,
                                   CONST double XC[], CONST double FVEC[],
                                   CONST double FJAC[],
                                   CONST Integer *LDFJAC, CONST double S[],
                                   CONST Integer *IGRADE,
                                   CONST Integer *NITER, CONST Integer *NF,
                                   Integer IW[], CONST Integer *LIW,
                                   double W[],
                                   CONST Integer *LW);

#ifdef __cplusplus
}
#endif

/*Declaration of global IUSER for this routine*/
Integer E04FCF_IUSER[6];

/* Length of array to hold error messages, no checks are made to ensure this is large enough */
#define LEN_ERR_MSG 100

void E04FCF_clean_up(double *X, double *FVEC, double *FJAC, double *S,
                     double *V, Integer *IW,
                     double *W);

SEXP WE04FCF(SEXP M, SEXP N, SEXP LSQFUN, SEXP LSQMON, SEXP IPRINT,
             SEXP MAXCAL, SEXP ETA, SEXP XTOL, SEXP STEPMX, SEXP X,
             SEXP RHO)
{
  Integer IFAIL_integer, IFAIL_EXTRACTION_integer;
  SEXP    ans, names;

/*initialization for input variables*/
  int     M_integer;
  if (nag_extractInteger(M, &M_integer))
    {
      error("Integer extraction error for %s", "M");
    }
  int     N_integer;
  if (nag_extractInteger(N, &N_integer))
    {
      error("Integer extraction error for %s", "N");
    }
  int     IPRINT_integer;
  if (nag_extractInteger(IPRINT, &IPRINT_integer))
    {
      error("Integer extraction error for %s", "IPRINT");
    }
  int     MAXCAL_integer;
  if (nag_extractInteger(MAXCAL, &MAXCAL_integer))
    {
      error("Integer extraction error for %s", "MAXCAL");
    }
  double  ETA_double;
  if (nag_extractDouble(ETA, &ETA_double))
    {
      error("Double extraction error for %s", "ETA");
    }
  double  XTOL_double;
  if (nag_extractDouble(XTOL, &XTOL_double))
    {
      error("Double extraction error for %s", "XTOL");
    }
  double  STEPMX_double;
  if (nag_extractDouble(STEPMX, &STEPMX_double))
    {
      error("Double extraction error for %s", "STEPMX");
    }
  double  *X_doubleMatrix = 0;

/*initialization for dimension variables*/
  Integer LDFJAC_integer = M_integer;
  Integer LDV_integer = N_integer;
  Integer LIW_integer = 1;
  Integer LW_integer;
  if (N_integer > 1)
    {
      LW_integer = 6*N_integer+M_integer*N_integer+2*M_integer+N_integer*
                   (N_integer-1) / 2;
    }
  else
    {
      if (N_integer == 1)
        {
          LW_integer = 7+3*M_integer;
        }
    }

/*initialization for output variables*/
  double  FSUMSQ_double;
  double  *FVEC_doubleMatrix = 0;
  double  *FJAC_doubleMatrix = 0;
  double  *S_doubleMatrix = 0;
  double  *V_doubleMatrix = 0;
  int     NITER_integer;
  int     NF_integer;

/*allocation general variables*/
  Integer X_dim1 = N_integer;
  if (!(X_doubleMatrix = (double *) malloc(X_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "X");
    }
  Integer FVEC_dim1 = M_integer;
  if (!(FVEC_doubleMatrix = (double *) malloc(FVEC_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "FVEC");
    }
  Integer FJAC_dim1 = LDFJAC_integer;
  Integer FJAC_dim2 = N_integer;
  if (!(FJAC_doubleMatrix =
          (double *) malloc(FJAC_dim1*FJAC_dim2*sizeof(double))))
    {
      error("Memory allocation error for %s", "FJAC");
    }
  Integer S_dim1 = N_integer;
  if (!(S_doubleMatrix = (double *) malloc(S_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "S");
    }
  Integer V_dim1 = LDV_integer;
  Integer V_dim2 = N_integer;
  if (!(V_doubleMatrix = (double *) malloc(V_dim1*V_dim2*sizeof(double))))
    {
      error("Memory allocation error for %s", "V");
    }

/*initialization for communication variables*/
  Integer *IW_integerMatrix = 0;
  double  *W_doubleMatrix = 0;

/*allocation communication variables*/
  Integer IW_dim1 = LIW_integer;
  if (!(IW_integerMatrix = (Integer *) malloc(IW_dim1*sizeof(Integer))))
    {
      error("Memory allocation error for %s", "IW");
    }
  Integer W_dim1 = LW_integer;
  if (!(W_doubleMatrix = (double *) malloc(W_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "W");
    }

/*allocation for dimension variables*/

/*extraction for input variables*/
  if ((IFAIL_EXTRACTION_integer =
         nag_extractDoubleMatrix(1, X_dim1, 1, X, X_doubleMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "X", nrows(X), X_dim1);
        }
      else
        {
          E04FCF_clean_up(X_doubleMatrix, FVEC_doubleMatrix, FJAC_doubleMatrix,
                          S_doubleMatrix, V_doubleMatrix, IW_integerMatrix,
                          W_doubleMatrix);
          error("Array extraction error for %s", "X");
        }
    }

/* Perform some initial checks */
  if (!isEnvironment(RHO))
    {
      error("rho must be an environment");
    }

  /* Get the maximum number of iterations, or use default of 100 if not supplied */
  if (MAXCAL_integer == NA_INTEGER) MAXCAL_integer = 100;
  /* Convert function pointer funct and rho into a couple of integers and store in IUSER */
  nag_convert_p2i(RHO, &E04FCF_IUSER[0]);
  nag_convert_p2i(LSQFUN, &E04FCF_IUSER[2]);
  nag_convert_p2i(LSQMON, &E04FCF_IUSER[4]);

  /* Call the NAG routine */
  IFAIL_integer = 1;
  e04fcf_(&M_integer, &N_integer, e04fcf_LSQFUN, e04fcf_LSQMON,
          &IPRINT_integer, &MAXCAL_integer, &ETA_double, &XTOL_double,
          &STEPMX_double,
          X_doubleMatrix, &FSUMSQ_double, FVEC_doubleMatrix, FJAC_doubleMatrix,
          &LDFJAC_integer, S_doubleMatrix, V_doubleMatrix, &LDV_integer,
          &NITER_integer,
          &NF_integer, IW_integerMatrix, &LIW_integer, W_doubleMatrix,
          &LW_integer,
          &IFAIL_integer);

  /* Error returns from NAG routine */
  if (IFAIL_integer != 0)
    {
      warning(
        "ABNORMAL EXIT from NAG library routine  E04FCF, : IFAIL = %ld\n",
        (long) IFAIL_integer);
    }

/* Get the list of results */
  PROTECT(ans = allocVector(VECSXP, 9));
  SET_VECTOR_ELT(ans, 0, nag_md2r(1, X_dim1, 1, X_doubleMatrix));
  SET_VECTOR_ELT(ans, 1, nag_d2r(1, &FSUMSQ_double));
  SET_VECTOR_ELT(ans, 2, nag_md2r(1, FVEC_dim1, 1, FVEC_doubleMatrix));
  SET_VECTOR_ELT(ans, 3, nag_md2r(1, FJAC_dim1, FJAC_dim2, FJAC_doubleMatrix));
  SET_VECTOR_ELT(ans, 4, nag_md2r(1, S_dim1, 1, S_doubleMatrix));
  SET_VECTOR_ELT(ans, 5, nag_md2r(1, V_dim1, V_dim2, V_doubleMatrix));
  SET_VECTOR_ELT(ans, 6, nag_i2r(1, &NITER_integer));
  SET_VECTOR_ELT(ans, 7, nag_i2r(1, &NF_integer));
  SET_VECTOR_ELT(ans, 8, nag_i2r(1, &IFAIL_integer));
  /* Get names for the list of results */
  PROTECT(names = allocVector(STRSXP, 9));
  SET_STRING_ELT(names, 0, mkChar("X"));
  SET_STRING_ELT(names, 1, mkChar("FSUMSQ"));
  SET_STRING_ELT(names, 2, mkChar("FVEC"));
  SET_STRING_ELT(names, 3, mkChar("FJAC"));
  SET_STRING_ELT(names, 4, mkChar("S"));
  SET_STRING_ELT(names, 5, mkChar("V"));
  SET_STRING_ELT(names, 6, mkChar("NITER"));
  SET_STRING_ELT(names, 7, mkChar("NF"));
  SET_STRING_ELT(names, 8, mkChar("IFAIL"));
  setAttrib(ans, R_NamesSymbol, names);
/* Unprotect the ans and names objects */
  UNPROTECT(2);

/*Call the clean-up routine*/
  E04FCF_clean_up(X_doubleMatrix, FVEC_doubleMatrix, FJAC_doubleMatrix,
                  S_doubleMatrix, V_doubleMatrix, IW_integerMatrix,
                  W_doubleMatrix);

  return(ans);
}

void NAG_CALL e04fcf_LSQFUN(Integer *IFLAG, CONST Integer *M, CONST Integer *N,
                            CONST double XC[], double FVEC[], Integer IW[],
                            CONST Integer *LIW, double W[],
                            CONST Integer *LW)
{
  SEXP funct, rho, R_fcall, result;
/* Get the pointer to the R environment */
  nag_convert_i2p(&E04FCF_IUSER[0], &rho); /* Get the pointer to the R function */
  nag_convert_i2p(&E04FCF_IUSER[2], &funct);

/* Give the arguments to the function */
  PROTECT(R_fcall = lang5(
            funct,
            nag_fi2r(1, IFLAG),
            nag_fi2r(1, M),
            nag_fi2r(1, N),
            nag_md2r(1, *N, 1, XC)));

/* Get the results from evaluating the function in R */
  PROTECT(result = eval(R_fcall, rho));
/* put the results returned by function into C variables */
  (void) nag_getIntFromList("IFLAG", result, IFLAG);
  (void) nag_getDoubleMatrixFromList("FVEC", 1, *M, 1, result, FVEC);

/* Release R_fcall */
  UNPROTECT(2);
}

void NAG_CALL e04fcf_LSQMON(CONST Integer *M, CONST Integer *N,
                            CONST double XC[], CONST double FVEC[],
                            CONST double FJAC[],
                            CONST Integer *LDFJAC, CONST double S[],
                            CONST Integer *IGRADE,
                            CONST Integer *NITER, CONST Integer *NF,
                            Integer IW[], CONST Integer *LIW,
                            double W[],
                            CONST Integer *LW)
{
  SEXP funct, rho, R_fcall, result;
/* Get the pointer to the R environment */
  nag_convert_i2p(&E04FCF_IUSER[0], &rho); /* Get the pointer to the R function */
  nag_convert_i2p(&E04FCF_IUSER[4], &funct);

/* Give the arguments to the function */
  PROTECT(R_fcall = lang11(
            funct,
            nag_fi2r(1, M),
            nag_fi2r(1, N),
            nag_md2r(1, *N, 1, XC),
            nag_md2r(1, *M, 1, FVEC),
            nag_md2r(1, *LDFJAC, *N, FJAC),
            nag_fi2r(1, LDFJAC),
            nag_md2r(1, *N, 1, S),
            nag_fi2r(1, IGRADE),
            nag_fi2r(1, NITER),
            nag_fi2r(1, NF)));

/* Get the results from evaluating the function in R */
  PROTECT(result = eval(R_fcall, rho));
/* put the results returned by function into C variables */

/* Release R_fcall */
  UNPROTECT(2);
}

void E04FCF_clean_up(double *X, double *FVEC, double *FJAC, double *S,
                     double *V, Integer *IW,
                     double *W)
{
  if (X) free(X);
  if (FVEC) free(FVEC);
  if (FJAC) free(FJAC);
  if (S) free(S);
  if (V) free(V);
  if (IW) free(IW);
  if (W) free(W);
  return;
}
