
#include <R.h>
#include <Rinternals.h>
#include "nag_rwrapper_utils.h"
#include "nagmk23.h"
#ifdef __cplusplus
extern "C" {
#endif
extern void NAG_CALL e04lbf_FUNCT(Integer *IFLAG, CONST Integer *N,
                                  CONST double XC[], double *FC, double GC[],
                                  Integer IW[], CONST Integer *LIW,
                                  double W[],
                                  CONST Integer *LW);
extern void NAG_CALL e04lbf_H(Integer *IFLAG, CONST Integer *N,
                              CONST double XC[], double FHESL[],
                              CONST Integer *LH, double FHESD[],
                              Integer IW[], CONST Integer *LIW, double W[],
                              CONST Integer *LW);
extern void NAG_CALL e04lbf_MONIT(CONST Integer *N, CONST double XC[],
                                  CONST double *FC, CONST double GC[],
                                  CONST Integer ISTATE[],
                                  CONST double *GPJNRM, CONST double *COND,
                                  CONST Integer *POSDEF,
                                  CONST Integer *NITER, CONST Integer *NF,
                                  Integer IW[], CONST Integer *LIW,
                                  double W[],
                                  CONST Integer *LW);

#ifdef __cplusplus
}
#endif

/*Declaration of global IUSER for this routine*/
Integer E04LBF_IUSER[8];

/* Length of array to hold error messages, no checks are made to ensure this is large enough */
#define LEN_ERR_MSG 100

void E04LBF_clean_up(double *BL, double *BU, double *X, double *HESL,
                     double *HESD, Integer *ISTATE, double *G, Integer *IW,
                     double *W);

SEXP WE04LBF(SEXP N, SEXP FUNCT, SEXP H, SEXP MONIT, SEXP IPRINT, SEXP MAXCAL,
             SEXP ETA, SEXP XTOL, SEXP STEPMX, SEXP IBOUND, SEXP BL, SEXP BU,
             SEXP X,
             SEXP LH, SEXP IW, SEXP W,
             SEXP rho)
{
  Integer IFAIL_integer;
  SEXP    ans, names;

/*initialization for input variables*/
  Integer N_integer = INTEGER(N)[0];
  Integer IPRINT_integer = INTEGER(IPRINT)[0];
  Integer MAXCAL_integer = INTEGER(MAXCAL)[0];
  double  ETA_double;
  ETA_double = REAL(ETA)[0];
  double  XTOL_double;
  XTOL_double = REAL(XTOL)[0];
  double  STEPMX_double;
  STEPMX_double = REAL(STEPMX)[0];
  Integer IBOUND_integer = INTEGER(IBOUND)[0];
  double  *BL_doubleMatrix = 0;
  double  *BU_doubleMatrix = 0;
  double  *X_doubleMatrix = 0;
  Integer LH_integer = INTEGER(LH)[0];

/*initialization for dimension variables*/
  Integer LIW_integer = 2;
  Integer LW_integer = MAX(7*N_integer+N_integer*(N_integer-1) / 2, 8);

/*initialization for output variables*/
  double  *HESL_doubleMatrix = 0;
  double  *HESD_doubleMatrix = 0;
  Integer *ISTATE_integerMatrix = 0;
  double  F_double;
  double  *G_doubleMatrix = 0;

/*allocation general variables*/
  Integer BL_dim1 = N_integer;
  Integer BL_dim2 = 1;
  if (!(BL_doubleMatrix = (double *) malloc(BL_dim1*sizeof(double))))
    {
      error("Memory allocation error");
    }
  Integer BU_dim1 = N_integer;
  Integer BU_dim2 = 1;
  if (!(BU_doubleMatrix = (double *) malloc(BU_dim1*sizeof(double))))
    {
      error("Memory allocation error");
    }
  Integer X_dim1 = N_integer;
  Integer X_dim2 = 1;
  if (!(X_doubleMatrix = (double *) malloc(X_dim1*sizeof(double))))
    {
      error("Memory allocation error");
    }
  Integer HESL_dim1 = LH_integer;
  Integer HESL_dim2 = 1;
  if (!(HESL_doubleMatrix = (double *) malloc(HESL_dim1*sizeof(double))))
    {
      error("Memory allocation error");
    }
  Integer HESD_dim1 = N_integer;
  Integer HESD_dim2 = 1;
  if (!(HESD_doubleMatrix = (double *) malloc(HESD_dim1*sizeof(double))))
    {
      error("Memory allocation error");
    }
  Integer ISTATE_dim1 = N_integer;
  Integer ISTATE_dim2 = 1;
  if (!(ISTATE_integerMatrix = (Integer *) malloc(ISTATE_dim1*sizeof(Integer))))
    {
      error("Memory allocation error");
    }
  Integer G_dim1 = N_integer;
  Integer G_dim2 = 1;
  if (!(G_doubleMatrix = (double *) malloc(G_dim1*sizeof(double))))
    {
      error("Memory allocation error");
    }

/*initialization for communication variables*/
  Integer *IW_integerMatrix = 0;
  double  *W_doubleMatrix = 0;

/*allocation communication variables*/
  Integer IW_dim1 = LIW_integer;
  if (!(IW_integerMatrix = (Integer *) malloc(IW_dim1*sizeof(Integer))))
    {
      error("Memory allocation error");
    }
  Integer W_dim1 = LW_integer;
  if (!(W_doubleMatrix = (double *) malloc(W_dim1*sizeof(double))))
    {
      error("Memory allocation error");
    }

/*allocation for dimension variables*/

/*extraction for input variables*/
  if (nag_extractDoubleMatrix(1, BL_dim1, BL_dim2, BL, BL_doubleMatrix))
    {
      E04LBF_clean_up(BL_doubleMatrix, BU_doubleMatrix, X_doubleMatrix,
                      HESL_doubleMatrix, HESD_doubleMatrix,
                      ISTATE_integerMatrix, G_doubleMatrix,
                      IW_integerMatrix,
                      W_doubleMatrix);
      error("Array extraction error for %s", "BL");
    }
  if (nag_extractDoubleMatrix(1, BU_dim1, BU_dim2, BU, BU_doubleMatrix))
    {
      E04LBF_clean_up(BL_doubleMatrix, BU_doubleMatrix, X_doubleMatrix,
                      HESL_doubleMatrix, HESD_doubleMatrix,
                      ISTATE_integerMatrix, G_doubleMatrix,
                      IW_integerMatrix,
                      W_doubleMatrix);
      error("Array extraction error for %s", "BU");
    }
  if (nag_extractDoubleMatrix(1, X_dim1, X_dim2, X, X_doubleMatrix))
    {
      E04LBF_clean_up(BL_doubleMatrix, BU_doubleMatrix, X_doubleMatrix,
                      HESL_doubleMatrix, HESD_doubleMatrix,
                      ISTATE_integerMatrix, G_doubleMatrix,
                      IW_integerMatrix,
                      W_doubleMatrix);
      error("Array extraction error for %s", "X");
    }

/* Perform some initial checks */
  if (!isFunction(FUNCT))
    {
      error("FUNCT must be a function");
    }
  if (!isFunction(H))
    {
      error("H must be a function");
    }
  if (!isFunction(MONIT))
    {
      error("MONIT must be a function");
    }
  if (!isEnvironment(rho))
    {
      error("rho must be an environment");
    }

  /* Get the maximum number of iterations, or use default of 100 if not supplied */
  if (MAXCAL_integer == NA_INTEGER) MAXCAL_integer = 100;
  /* Convert function pointer funct and rho into a couple of integers and store in IUSER */
  nag_convert_p2i(rho, &E04LBF_IUSER[0]);
  nag_convert_p2i(FUNCT, &E04LBF_IUSER[2]);
  nag_convert_p2i(H, &E04LBF_IUSER[4]);
  nag_convert_p2i(MONIT, &E04LBF_IUSER[6]);

  /* Call the NAG routine */
  IFAIL_integer = 1;
  e04lbf_(&N_integer, e04lbf_FUNCT, e04lbf_H, e04lbf_MONIT, &IPRINT_integer,
          &MAXCAL_integer, &ETA_double, &XTOL_double, &STEPMX_double,
          &IBOUND_integer,
          BL_doubleMatrix, BU_doubleMatrix, X_doubleMatrix, HESL_doubleMatrix,
          &LH_integer, HESD_doubleMatrix, ISTATE_integerMatrix, &F_double,
          G_doubleMatrix, IW_integerMatrix, &LIW_integer, W_doubleMatrix,
          &LW_integer,
          &IFAIL_integer);

  /* Error returns from NAG routine */
  if (IFAIL_integer != 0)
    {
      if (IFAIL_integer == 3)
        {
          warning(
            "WARNING on EXIT from NAG library routine  E04LBF, : IFAIL = %ld\n",
            (long) IFAIL_integer);
        }
      else
        {
          warning(
            "ABNORMAL EXIT from NAG library routine  E04LBF, : IFAIL = %ld\n",
            (long) IFAIL_integer);
        }
    }

/* Get the list of results */
  PROTECT(ans = allocVector(VECSXP, 9));
  SET_VECTOR_ELT(ans, 0, nag_md2r(1, BL_dim1, BL_dim2, BL_doubleMatrix));
  SET_VECTOR_ELT(ans, 1, nag_md2r(1, BU_dim1, BU_dim2, BU_doubleMatrix));
  SET_VECTOR_ELT(ans, 2, nag_md2r(1, X_dim1, X_dim2, X_doubleMatrix));
  SET_VECTOR_ELT(ans, 3, nag_md2r(1, HESL_dim1, HESL_dim2, HESL_doubleMatrix));
  SET_VECTOR_ELT(ans, 4, nag_md2r(1, HESD_dim1, HESD_dim2, HESD_doubleMatrix));
  SET_VECTOR_ELT(ans, 5,
                 nag_mi2r(1, ISTATE_dim1, ISTATE_dim2, ISTATE_integerMatrix));
  SET_VECTOR_ELT(ans, 6, nag_d2r(1, &F_double));
  SET_VECTOR_ELT(ans, 7, nag_md2r(1, G_dim1, G_dim2, G_doubleMatrix));
  SET_VECTOR_ELT(ans, 8, nag_i2r(1, &IFAIL_integer));
  /* Get names for the list of results */
  PROTECT(names = allocVector(STRSXP, 9));
  SET_STRING_ELT(names, 0, mkChar("BL"));
  SET_STRING_ELT(names, 1, mkChar("BU"));
  SET_STRING_ELT(names, 2, mkChar("X"));
  SET_STRING_ELT(names, 3, mkChar("HESL"));
  SET_STRING_ELT(names, 4, mkChar("HESD"));
  SET_STRING_ELT(names, 5, mkChar("ISTATE"));
  SET_STRING_ELT(names, 6, mkChar("F"));
  SET_STRING_ELT(names, 7, mkChar("G"));
  SET_STRING_ELT(names, 8, mkChar("IFAIL"));
  setAttrib(ans, R_NamesSymbol, names);
/* Unprotect the ans and names objects */
  UNPROTECT(2);

/*Call the clean-up routine*/
  E04LBF_clean_up(BL_doubleMatrix, BU_doubleMatrix, X_doubleMatrix,
                  HESL_doubleMatrix, HESD_doubleMatrix, ISTATE_integerMatrix,
                  G_doubleMatrix,
                  IW_integerMatrix,
                  W_doubleMatrix);

  return(ans);
}

void NAG_CALL e04lbf_FUNCT(Integer *IFLAG, CONST Integer *N, CONST double XC[],
                           double *FC, double GC[], Integer IW[],
                           CONST Integer *LIW, double W[],
                           CONST Integer *LW)
{
  SEXP funct, rho, R_fcall, result;
/* Get the pointer to the R environment */
  nag_convert_i2p(&E04LBF_IUSER[0], &rho); /* Get the pointer to the R function */
  nag_convert_i2p(&E04LBF_IUSER[2], &funct);

/* Give the arguments to the function */
  PROTECT(R_fcall = lang6(
            funct,
            nag_fi2r(1, IFLAG),
            nag_fi2r(1, N),
            nag_md2r(1, *N, 1, XC),
            nag_d2r(1, FC),
            nag_md2r(1, *N, 1, GC)));

/* Get the results from evaluating the function in R */
  PROTECT(result = eval(R_fcall, rho));
/* put the results returned by function into C variables */
  (void) nag_getIntFromList("IFLAG", result, IFLAG);
  (void) nag_getDoubleScalarFromList("FC", result, FC);
  (void) nag_getDoubleMatrixFromList("GC", 1, *N, 1, result, GC);

/* Release R_fcall */
  UNPROTECT(2);
}

void NAG_CALL e04lbf_H(Integer *IFLAG, CONST Integer *N, CONST double XC[],
                       double FHESL[], CONST Integer *LH, double FHESD[],
                       Integer IW[],
                       CONST Integer *LIW, double W[],
                       CONST Integer *LW)
{
  SEXP funct, rho, R_fcall, result;
/* Get the pointer to the R environment */
  nag_convert_i2p(&E04LBF_IUSER[0], &rho); /* Get the pointer to the R function */
  nag_convert_i2p(&E04LBF_IUSER[4], &funct);

/* Give the arguments to the function */
  PROTECT(R_fcall = lang7(
            funct,
            nag_fi2r(1, IFLAG),
            nag_fi2r(1, N),
            nag_md2r(1, *N, 1, XC),
            nag_md2r(1, *LH, 1, FHESL),
            nag_fi2r(1, LH),
            nag_md2r(1, *N, 1, FHESD)));

/* Get the results from evaluating the function in R */
  PROTECT(result = eval(R_fcall, rho));
/* put the results returned by function into C variables */
  (void) nag_getIntFromList("IFLAG", result, IFLAG);
  (void) nag_getDoubleMatrixFromList("FHESL", 1, *LH, 1, result, FHESL);
  (void) nag_getDoubleMatrixFromList("FHESD", 1, *N, 1, result, FHESD);

/* Release R_fcall */
  UNPROTECT(2);
}

void NAG_CALL e04lbf_MONIT(CONST Integer *N, CONST double XC[],
                           CONST double *FC, CONST double GC[],
                           CONST Integer ISTATE[],
                           CONST double *GPJNRM, CONST double *COND,
                           CONST Integer *POSDEF,
                           CONST Integer *NITER, CONST Integer *NF,
                           Integer IW[], CONST Integer *LIW,
                           double W[],
                           CONST Integer *LW)
{
  SEXP funct, rho, R_fcall, result;
/* Get the pointer to the R environment */
  nag_convert_i2p(&E04LBF_IUSER[0], &rho); /* Get the pointer to the R function */
  nag_convert_i2p(&E04LBF_IUSER[6], &funct);

/* Give the arguments to the function */
  PROTECT(R_fcall = lang11(
            funct,
            nag_fi2r(1, N),
            nag_md2r(1, *N, 1, XC),
            nag_d2r(1, FC),
            nag_md2r(1, *N, 1, GC),
            nag_fi2r(*N, ISTATE),
            nag_d2r(1, GPJNRM),
            nag_d2r(1, COND),
            nag_fi2r(1, POSDEF),
            nag_fi2r(1, NITER),
            nag_fi2r(1, NF)));

/* Get the results from evaluating the function in R */
  PROTECT(result = eval(R_fcall, rho));
/* put the results returned by function into C variables */

/* Release R_fcall */
  UNPROTECT(2);
}

void E04LBF_clean_up(double *BL, double *BU, double *X, double *HESL,
                     double *HESD, Integer *ISTATE, double *G, Integer *IW,
                     double *W)
{
  if (BL) free(BL);
  if (BU) free(BU);
  if (X) free(X);
  if (HESL) free(HESL);
  if (HESD) free(HESD);
  if (ISTATE) free(ISTATE);
  if (G) free(G);
  if (IW) free(IW);
  if (W) free(W);
  return;
}
