
#include <R.h>
#include <Rinternals.h>
#include "nag_rwrapper_utils.h"
#include "nagmk23.h"
#ifdef __cplusplus
extern "C" {
#endif
extern void NAG_CALL e04vhf_USRFUN(Integer *STATUS, CONST Integer *N,
                                   CONST double X[], CONST Integer *NEEDF,
                                   CONST Integer *NF, double F[],
                                   CONST Integer *NEEDG, CONST Integer *LENG,
                                   double G[], char CUSER[],
                                   Integer *E04VHF_IUSER,
                                   double E04VHF_RUSER[],
                                   CONST int CUSER_dim);

#ifdef __cplusplus
}
#endif
int wE04VHF_handle_options(SEXP OPTLIST, char CW_charArray[],
                           Integer IW_integerMatrix[],
                           double RW_doubleMatrix[],
                           CONST int CW_dim)
;

/* Length of array to hold error messages, no checks are made to ensure this is large enough */
#define LEN_ERR_MSG 100

void E04VHF_clean_up(Integer *IAFUN, Integer *JAVAR, double *A, Integer *IGFUN,
                     Integer *JGVAR, double *XLOW, double *XUPP, char *XNAMES,
                     double *FLOW,
                     double *FUPP, char *FNAMES, double *X, Integer *XSTATE,
                     double *XMUL, double *F,
                     Integer *FSTATE, double *FMUL, char *CW, Integer *IW,
                     double *RW);

SEXP WE04VHF(SEXP START, SEXP NF, SEXP N, SEXP NXNAME, SEXP NFNAME,
             SEXP OBJADD, SEXP OBJROW, SEXP PROB, SEXP USRFUN, SEXP IAFUN,
             SEXP JAVAR,
             SEXP LENA, SEXP NEA, SEXP A, SEXP IGFUN, SEXP JGVAR, SEXP LENG,
             SEXP NEG,
             SEXP XLOW, SEXP XUPP, SEXP XNAMES, SEXP FLOW, SEXP FUPP,
             SEXP FNAMES, SEXP X,
             SEXP XSTATE, SEXP F, SEXP FSTATE, SEXP FMUL, SEXP NS,
             SEXP OPTLIST,
             SEXP RHO)
{
  Integer IFAIL_integer, IFAIL_EXTRACTION_integer;
  Integer E04VHF_IUSER[4];
  double  E04VHF_RUSER[1];
  char    CUSER_charArray[1];

  SEXP    ans, names;
  Integer nadv, iflag;

  /* Initialize the option arrays */
  Integer IFAIL_initializer_integer = 1;
  char    err_msg[LEN_ERR_MSG];

/*Declare dimension variables for option initializer*/
  Integer LENCW_integer = 600;
  Integer LENIW_integer = 600;
  Integer LENRW_integer = 600;

/*Declare the parameters for the option initializer*/
  char    *CW_charArray = 0;
  Integer *IW_integerMatrix = 0;
  double  *RW_doubleMatrix = 0;
  Integer CW_dim1 = LENCW_integer;
  if (!(CW_charArray = (char *) malloc(CW_dim1*8*sizeof(char))))
    {
      error("Memory allocation error for %s", "CW");
    }
  Integer IW_dim1 = LENIW_integer;
  if (!(IW_integerMatrix = (Integer *) malloc(IW_dim1*sizeof(Integer))))
    {
      error("Memory allocation error for %s", "IW");
    }
  Integer RW_dim1 = LENRW_integer;
  if (!(RW_doubleMatrix = (double *) malloc(RW_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "RW");
    }

  nag_initializeIntegerMatrix(IW_dim1, 1, IW_integerMatrix);
  nag_initializeDoubleMatrix(RW_dim1, 1, RW_doubleMatrix);
  e04vgf_(CW_charArray, &LENCW_integer, IW_integerMatrix, &LENIW_integer,
          RW_doubleMatrix, &LENRW_integer, &IFAIL_initializer_integer,
          8);

  if (IFAIL_initializer_integer != 0)
    {
      sprintf(err_msg, "E04VGF failed with IFAIL = %ld\n",
              (long) IFAIL_initializer_integer);
      error(err_msg);
    } /* Handle options */
  int     unit_num;

  unit_num =
    wE04VHF_handle_options(OPTLIST, CW_charArray, IW_integerMatrix,
                           RW_doubleMatrix,
                           8);
  if (unit_num < 0)
    {
      /* Error opening output file */
      sprintf(
        err_msg,
        "Error opening monitoring file for output, check value for `monitoring file` entry in `optlist`");
      error(err_msg);
    }
  else if (unit_num > 0)
    {
      /* We are using an output file, so need to handle the advisory channel */

      /* Remember the current value */
      iflag = 0;
      x04abf_(&iflag, &nadv);

      /* Set the advisory channel to the output file. Need to do this, otherwise E04VHF sends output to both the
         advisory channel and the output file */
      iflag = 1;
      x04abf_(&iflag, &unit_num);
    }

/*initialization for input variables*/
  int        START_integer;
  if (nag_extractInteger(START, &START_integer))
    {
      error("Integer extraction error for %s", "START");
    }
  int        NF_integer;
  if (nag_extractInteger(NF, &NF_integer))
    {
      error("Integer extraction error for %s", "NF");
    }
  int        N_integer;
  if (nag_extractInteger(N, &N_integer))
    {
      error("Integer extraction error for %s", "N");
    }
  int        NXNAME_integer;
  if (nag_extractInteger(NXNAME, &NXNAME_integer))
    {
      error("Integer extraction error for %s", "NXNAME");
    }
  int        NFNAME_integer;
  if (nag_extractInteger(NFNAME, &NFNAME_integer))
    {
      error("Integer extraction error for %s", "NFNAME");
    }
  double     OBJADD_double;
  if (nag_extractDouble(OBJADD, &OBJADD_double))
    {
      error("Double extraction error for %s", "OBJADD");
    }
  int        OBJROW_integer;
  const char *PROB_char = CHAR(STRING_ELT(PROB, 0));
  Integer    PROB_length = 1;
  Integer    *IAFUN_integerMatrix = 0;
  Integer    *JAVAR_integerMatrix = 0;
  int        LENA_integer;
  if (nag_extractInteger(LENA, &LENA_integer))
    {
      error("Integer extraction error for %s", "LENA");
    }
  int        NEA_integer;
  if (nag_extractInteger(NEA, &NEA_integer))
    {
      error("Integer extraction error for %s", "NEA");
    }
  double     *A_doubleMatrix = 0;
  Integer    *IGFUN_integerMatrix = 0;
  Integer    *JGVAR_integerMatrix = 0;
  int        LENG_integer;
  if (nag_extractInteger(LENG, &LENG_integer))
    {
      error("Integer extraction error for %s", "LENG");
    }
  int        NEG_integer;
  if (nag_extractInteger(NEG, &NEG_integer))
    {
      error("Integer extraction error for %s", "NEG");
    }
  double     *XLOW_doubleMatrix = 0;
  double     *XUPP_doubleMatrix = 0;
  char       *XNAMES_charArray = 0;
  double     *FLOW_doubleMatrix = 0;
  double     *FUPP_doubleMatrix = 0;
  char       *FNAMES_charArray = 0;
  double     *X_doubleMatrix = 0;
  Integer    *XSTATE_integerMatrix = 0;
  double     *F_doubleMatrix = 0;
  Integer    *FSTATE_integerMatrix = 0;
  double     *FMUL_doubleMatrix = 0;
  int        NS_integer;
  if (nag_extractInteger(NS, &NS_integer))
    {
      error("Integer extraction error for %s", "NS");
    }

/*initialization for output variables*/
  double     *XMUL_doubleMatrix = 0;
  int        NINF_integer;
  double     SINF_double;

/*allocation general variables*/
  Integer    IAFUN_dim1 = LENA_integer;
  if (!(IAFUN_integerMatrix = (Integer *) malloc(IAFUN_dim1*sizeof(Integer))))
    {
      error("Memory allocation error for %s", "IAFUN");
    }
  Integer    JAVAR_dim1 = LENA_integer;
  if (!(JAVAR_integerMatrix = (Integer *) malloc(JAVAR_dim1*sizeof(Integer))))
    {
      error("Memory allocation error for %s", "JAVAR");
    }
  Integer    A_dim1 = LENA_integer;
  if (!(A_doubleMatrix = (double *) malloc(A_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "A");
    }
  Integer    IGFUN_dim1 = LENG_integer;
  if (!(IGFUN_integerMatrix = (Integer *) malloc(IGFUN_dim1*sizeof(Integer))))
    {
      error("Memory allocation error for %s", "IGFUN");
    }
  Integer    JGVAR_dim1 = LENG_integer;
  if (!(JGVAR_integerMatrix = (Integer *) malloc(JGVAR_dim1*sizeof(Integer))))
    {
      error("Memory allocation error for %s", "JGVAR");
    }
  Integer    XLOW_dim1 = N_integer;
  if (!(XLOW_doubleMatrix = (double *) malloc(XLOW_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "XLOW");
    }
  Integer    XUPP_dim1 = N_integer;
  if (!(XUPP_doubleMatrix = (double *) malloc(XUPP_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "XUPP");
    }
  Integer    XNAMES_dim1 = NXNAME_integer;
  if (!(XNAMES_charArray = (char *) malloc(XNAMES_dim1*8*sizeof(char))))
    {
      error("Memory allocation error for %s", "XNAMES");
    }
  Integer    FLOW_dim1 = NF_integer;
  if (!(FLOW_doubleMatrix = (double *) malloc(FLOW_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "FLOW");
    }
  Integer    FUPP_dim1 = NF_integer;
  if (!(FUPP_doubleMatrix = (double *) malloc(FUPP_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "FUPP");
    }
  Integer    FNAMES_dim1 = NFNAME_integer;
  if (!(FNAMES_charArray = (char *) malloc(FNAMES_dim1*8*sizeof(char))))
    {
      error("Memory allocation error for %s", "FNAMES");
    }
  Integer    X_dim1 = N_integer;
  if (!(X_doubleMatrix = (double *) malloc(X_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "X");
    }
  Integer    XSTATE_dim1 = N_integer;
  if (!(XSTATE_integerMatrix = (Integer *) malloc(XSTATE_dim1*sizeof(Integer))))
    {
      error("Memory allocation error for %s", "XSTATE");
    }
  Integer    XMUL_dim1 = N_integer;
  if (!(XMUL_doubleMatrix = (double *) malloc(XMUL_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "XMUL");
    }
  Integer    F_dim1 = NF_integer;
  if (!(F_doubleMatrix = (double *) malloc(F_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "F");
    }
  Integer    FSTATE_dim1 = NF_integer;
  if (!(FSTATE_integerMatrix = (Integer *) malloc(FSTATE_dim1*sizeof(Integer))))
    {
      error("Memory allocation error for %s", "FSTATE");
    }
  Integer    FMUL_dim1 = NF_integer;
  if (!(FMUL_doubleMatrix = (double *) malloc(FMUL_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "FMUL");
    }

/*allocation for dimension variables*/

/*extraction for input variables*/
  if ((IFAIL_EXTRACTION_integer =
         nag_extractIntegerMatrix(1, IAFUN_dim1, 1, IAFUN, IAFUN_integerMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "IAFUN", nrows(IAFUN), IAFUN_dim1);
        }
      else
        {
          E04VHF_clean_up(IAFUN_integerMatrix, JAVAR_integerMatrix,
                          A_doubleMatrix, IGFUN_integerMatrix,
                          JGVAR_integerMatrix,
                          XLOW_doubleMatrix, XUPP_doubleMatrix,
                          XNAMES_charArray, FLOW_doubleMatrix,
                          FUPP_doubleMatrix, FNAMES_charArray, X_doubleMatrix,
                          XSTATE_integerMatrix,
                          XMUL_doubleMatrix, F_doubleMatrix,
                          FSTATE_integerMatrix,
                          FMUL_doubleMatrix, CW_charArray, IW_integerMatrix,
                          RW_doubleMatrix);
          error("Array extraction error for %s", "IAFUN");
        }
    }
  if ((IFAIL_EXTRACTION_integer =
         nag_extractIntegerMatrix(1, JAVAR_dim1, 1, JAVAR, JAVAR_integerMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "JAVAR", nrows(JAVAR), JAVAR_dim1);
        }
      else
        {
          E04VHF_clean_up(IAFUN_integerMatrix, JAVAR_integerMatrix,
                          A_doubleMatrix, IGFUN_integerMatrix,
                          JGVAR_integerMatrix,
                          XLOW_doubleMatrix, XUPP_doubleMatrix,
                          XNAMES_charArray, FLOW_doubleMatrix,
                          FUPP_doubleMatrix, FNAMES_charArray, X_doubleMatrix,
                          XSTATE_integerMatrix,
                          XMUL_doubleMatrix, F_doubleMatrix,
                          FSTATE_integerMatrix,
                          FMUL_doubleMatrix, CW_charArray, IW_integerMatrix,
                          RW_doubleMatrix);
          error("Array extraction error for %s", "JAVAR");
        }
    }
  if ((IFAIL_EXTRACTION_integer =
         nag_extractDoubleMatrix(1, A_dim1, 1, A, A_doubleMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "A", nrows(A), A_dim1);
        }
      else
        {
          E04VHF_clean_up(IAFUN_integerMatrix, JAVAR_integerMatrix,
                          A_doubleMatrix, IGFUN_integerMatrix,
                          JGVAR_integerMatrix,
                          XLOW_doubleMatrix, XUPP_doubleMatrix,
                          XNAMES_charArray, FLOW_doubleMatrix,
                          FUPP_doubleMatrix, FNAMES_charArray, X_doubleMatrix,
                          XSTATE_integerMatrix,
                          XMUL_doubleMatrix, F_doubleMatrix,
                          FSTATE_integerMatrix,
                          FMUL_doubleMatrix, CW_charArray, IW_integerMatrix,
                          RW_doubleMatrix);
          error("Array extraction error for %s", "A");
        }
    }
  if ((IFAIL_EXTRACTION_integer =
         nag_extractIntegerMatrix(1, IGFUN_dim1, 1, IGFUN, IGFUN_integerMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "IGFUN", nrows(IGFUN), IGFUN_dim1);
        }
      else
        {
          E04VHF_clean_up(IAFUN_integerMatrix, JAVAR_integerMatrix,
                          A_doubleMatrix, IGFUN_integerMatrix,
                          JGVAR_integerMatrix,
                          XLOW_doubleMatrix, XUPP_doubleMatrix,
                          XNAMES_charArray, FLOW_doubleMatrix,
                          FUPP_doubleMatrix, FNAMES_charArray, X_doubleMatrix,
                          XSTATE_integerMatrix,
                          XMUL_doubleMatrix, F_doubleMatrix,
                          FSTATE_integerMatrix,
                          FMUL_doubleMatrix, CW_charArray, IW_integerMatrix,
                          RW_doubleMatrix);
          error("Array extraction error for %s", "IGFUN");
        }
    }
  if ((IFAIL_EXTRACTION_integer =
         nag_extractIntegerMatrix(1, JGVAR_dim1, 1, JGVAR, JGVAR_integerMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "JGVAR", nrows(JGVAR), JGVAR_dim1);
        }
      else
        {
          E04VHF_clean_up(IAFUN_integerMatrix, JAVAR_integerMatrix,
                          A_doubleMatrix, IGFUN_integerMatrix,
                          JGVAR_integerMatrix,
                          XLOW_doubleMatrix, XUPP_doubleMatrix,
                          XNAMES_charArray, FLOW_doubleMatrix,
                          FUPP_doubleMatrix, FNAMES_charArray, X_doubleMatrix,
                          XSTATE_integerMatrix,
                          XMUL_doubleMatrix, F_doubleMatrix,
                          FSTATE_integerMatrix,
                          FMUL_doubleMatrix, CW_charArray, IW_integerMatrix,
                          RW_doubleMatrix);
          error("Array extraction error for %s", "JGVAR");
        }
    }
  if ((IFAIL_EXTRACTION_integer =
         nag_extractDoubleMatrix(1, XLOW_dim1, 1, XLOW, XLOW_doubleMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "XLOW", nrows(XLOW), XLOW_dim1);
        }
      else
        {
          E04VHF_clean_up(IAFUN_integerMatrix, JAVAR_integerMatrix,
                          A_doubleMatrix, IGFUN_integerMatrix,
                          JGVAR_integerMatrix,
                          XLOW_doubleMatrix, XUPP_doubleMatrix,
                          XNAMES_charArray, FLOW_doubleMatrix,
                          FUPP_doubleMatrix, FNAMES_charArray, X_doubleMatrix,
                          XSTATE_integerMatrix,
                          XMUL_doubleMatrix, F_doubleMatrix,
                          FSTATE_integerMatrix,
                          FMUL_doubleMatrix, CW_charArray, IW_integerMatrix,
                          RW_doubleMatrix);
          error("Array extraction error for %s", "XLOW");
        }
    }
  if ((IFAIL_EXTRACTION_integer =
         nag_extractDoubleMatrix(1, XUPP_dim1, 1, XUPP, XUPP_doubleMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "XUPP", nrows(XUPP), XUPP_dim1);
        }
      else
        {
          E04VHF_clean_up(IAFUN_integerMatrix, JAVAR_integerMatrix,
                          A_doubleMatrix, IGFUN_integerMatrix,
                          JGVAR_integerMatrix,
                          XLOW_doubleMatrix, XUPP_doubleMatrix,
                          XNAMES_charArray, FLOW_doubleMatrix,
                          FUPP_doubleMatrix, FNAMES_charArray, X_doubleMatrix,
                          XSTATE_integerMatrix,
                          XMUL_doubleMatrix, F_doubleMatrix,
                          FSTATE_integerMatrix,
                          FMUL_doubleMatrix, CW_charArray, IW_integerMatrix,
                          RW_doubleMatrix);
          error("Array extraction error for %s", "XUPP");
        }
    }
  if ((IFAIL_EXTRACTION_integer =
         nag_extractCharVector(XNAMES_dim1, XNAMES, XNAMES_charArray, 8)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "XNAMES", nrows(XNAMES), XNAMES_dim1);
        }
      else
        {
          E04VHF_clean_up(IAFUN_integerMatrix, JAVAR_integerMatrix,
                          A_doubleMatrix, IGFUN_integerMatrix,
                          JGVAR_integerMatrix,
                          XLOW_doubleMatrix, XUPP_doubleMatrix,
                          XNAMES_charArray, FLOW_doubleMatrix,
                          FUPP_doubleMatrix, FNAMES_charArray, X_doubleMatrix,
                          XSTATE_integerMatrix,
                          XMUL_doubleMatrix, F_doubleMatrix,
                          FSTATE_integerMatrix,
                          FMUL_doubleMatrix, CW_charArray, IW_integerMatrix,
                          RW_doubleMatrix);
          error("Array extraction error for %s", "XNAMES");
        }
    }
  if ((IFAIL_EXTRACTION_integer =
         nag_extractDoubleMatrix(1, FLOW_dim1, 1, FLOW, FLOW_doubleMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "FLOW", nrows(FLOW), FLOW_dim1);
        }
      else
        {
          E04VHF_clean_up(IAFUN_integerMatrix, JAVAR_integerMatrix,
                          A_doubleMatrix, IGFUN_integerMatrix,
                          JGVAR_integerMatrix,
                          XLOW_doubleMatrix, XUPP_doubleMatrix,
                          XNAMES_charArray, FLOW_doubleMatrix,
                          FUPP_doubleMatrix, FNAMES_charArray, X_doubleMatrix,
                          XSTATE_integerMatrix,
                          XMUL_doubleMatrix, F_doubleMatrix,
                          FSTATE_integerMatrix,
                          FMUL_doubleMatrix, CW_charArray, IW_integerMatrix,
                          RW_doubleMatrix);
          error("Array extraction error for %s", "FLOW");
        }
    }
  if ((IFAIL_EXTRACTION_integer =
         nag_extractDoubleMatrix(1, FUPP_dim1, 1, FUPP, FUPP_doubleMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "FUPP", nrows(FUPP), FUPP_dim1);
        }
      else
        {
          E04VHF_clean_up(IAFUN_integerMatrix, JAVAR_integerMatrix,
                          A_doubleMatrix, IGFUN_integerMatrix,
                          JGVAR_integerMatrix,
                          XLOW_doubleMatrix, XUPP_doubleMatrix,
                          XNAMES_charArray, FLOW_doubleMatrix,
                          FUPP_doubleMatrix, FNAMES_charArray, X_doubleMatrix,
                          XSTATE_integerMatrix,
                          XMUL_doubleMatrix, F_doubleMatrix,
                          FSTATE_integerMatrix,
                          FMUL_doubleMatrix, CW_charArray, IW_integerMatrix,
                          RW_doubleMatrix);
          error("Array extraction error for %s", "FUPP");
        }
    }
  if ((IFAIL_EXTRACTION_integer =
         nag_extractCharVector(FNAMES_dim1, FNAMES, FNAMES_charArray, 8)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "FNAMES", nrows(FNAMES), FNAMES_dim1);
        }
      else
        {
          E04VHF_clean_up(IAFUN_integerMatrix, JAVAR_integerMatrix,
                          A_doubleMatrix, IGFUN_integerMatrix,
                          JGVAR_integerMatrix,
                          XLOW_doubleMatrix, XUPP_doubleMatrix,
                          XNAMES_charArray, FLOW_doubleMatrix,
                          FUPP_doubleMatrix, FNAMES_charArray, X_doubleMatrix,
                          XSTATE_integerMatrix,
                          XMUL_doubleMatrix, F_doubleMatrix,
                          FSTATE_integerMatrix,
                          FMUL_doubleMatrix, CW_charArray, IW_integerMatrix,
                          RW_doubleMatrix);
          error("Array extraction error for %s", "FNAMES");
        }
    }
  if ((IFAIL_EXTRACTION_integer =
         nag_extractDoubleMatrix(1, X_dim1, 1, X, X_doubleMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "X", nrows(X), X_dim1);
        }
      else
        {
          E04VHF_clean_up(IAFUN_integerMatrix, JAVAR_integerMatrix,
                          A_doubleMatrix, IGFUN_integerMatrix,
                          JGVAR_integerMatrix,
                          XLOW_doubleMatrix, XUPP_doubleMatrix,
                          XNAMES_charArray, FLOW_doubleMatrix,
                          FUPP_doubleMatrix, FNAMES_charArray, X_doubleMatrix,
                          XSTATE_integerMatrix,
                          XMUL_doubleMatrix, F_doubleMatrix,
                          FSTATE_integerMatrix,
                          FMUL_doubleMatrix, CW_charArray, IW_integerMatrix,
                          RW_doubleMatrix);
          error("Array extraction error for %s", "X");
        }
    }
  if ((IFAIL_EXTRACTION_integer =
         nag_extractIntegerMatrix(1, XSTATE_dim1, 1, XSTATE,
                                  XSTATE_integerMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "XSTATE", nrows(XSTATE), XSTATE_dim1);
        }
      else
        {
          E04VHF_clean_up(IAFUN_integerMatrix, JAVAR_integerMatrix,
                          A_doubleMatrix, IGFUN_integerMatrix,
                          JGVAR_integerMatrix,
                          XLOW_doubleMatrix, XUPP_doubleMatrix,
                          XNAMES_charArray, FLOW_doubleMatrix,
                          FUPP_doubleMatrix, FNAMES_charArray, X_doubleMatrix,
                          XSTATE_integerMatrix,
                          XMUL_doubleMatrix, F_doubleMatrix,
                          FSTATE_integerMatrix,
                          FMUL_doubleMatrix, CW_charArray, IW_integerMatrix,
                          RW_doubleMatrix);
          error("Array extraction error for %s", "XSTATE");
        }
    }
  if ((IFAIL_EXTRACTION_integer =
         nag_extractDoubleMatrix(1, F_dim1, 1, F, F_doubleMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "F", nrows(F), F_dim1);
        }
      else
        {
          E04VHF_clean_up(IAFUN_integerMatrix, JAVAR_integerMatrix,
                          A_doubleMatrix, IGFUN_integerMatrix,
                          JGVAR_integerMatrix,
                          XLOW_doubleMatrix, XUPP_doubleMatrix,
                          XNAMES_charArray, FLOW_doubleMatrix,
                          FUPP_doubleMatrix, FNAMES_charArray, X_doubleMatrix,
                          XSTATE_integerMatrix,
                          XMUL_doubleMatrix, F_doubleMatrix,
                          FSTATE_integerMatrix,
                          FMUL_doubleMatrix, CW_charArray, IW_integerMatrix,
                          RW_doubleMatrix);
          error("Array extraction error for %s", "F");
        }
    }
  if ((IFAIL_EXTRACTION_integer =
         nag_extractIntegerMatrix(1, FSTATE_dim1, 1, FSTATE,
                                  FSTATE_integerMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "FSTATE", nrows(FSTATE), FSTATE_dim1);
        }
      else
        {
          E04VHF_clean_up(IAFUN_integerMatrix, JAVAR_integerMatrix,
                          A_doubleMatrix, IGFUN_integerMatrix,
                          JGVAR_integerMatrix,
                          XLOW_doubleMatrix, XUPP_doubleMatrix,
                          XNAMES_charArray, FLOW_doubleMatrix,
                          FUPP_doubleMatrix, FNAMES_charArray, X_doubleMatrix,
                          XSTATE_integerMatrix,
                          XMUL_doubleMatrix, F_doubleMatrix,
                          FSTATE_integerMatrix,
                          FMUL_doubleMatrix, CW_charArray, IW_integerMatrix,
                          RW_doubleMatrix);
          error("Array extraction error for %s", "FSTATE");
        }
    }
  if ((IFAIL_EXTRACTION_integer =
         nag_extractDoubleMatrix(1, FMUL_dim1, 1, FMUL, FMUL_doubleMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "FMUL", nrows(FMUL), FMUL_dim1);
        }
      else
        {
          E04VHF_clean_up(IAFUN_integerMatrix, JAVAR_integerMatrix,
                          A_doubleMatrix, IGFUN_integerMatrix,
                          JGVAR_integerMatrix,
                          XLOW_doubleMatrix, XUPP_doubleMatrix,
                          XNAMES_charArray, FLOW_doubleMatrix,
                          FUPP_doubleMatrix, FNAMES_charArray, X_doubleMatrix,
                          XSTATE_integerMatrix,
                          XMUL_doubleMatrix, F_doubleMatrix,
                          FSTATE_integerMatrix,
                          FMUL_doubleMatrix, CW_charArray, IW_integerMatrix,
                          RW_doubleMatrix);
          error("Array extraction error for %s", "FMUL");
        }
    }

/* Perform some initial checks */
  if (!isEnvironment(RHO))
    {
      error("rho must be an environment");
    }

  /* Convert function pointer funct and rho into a couple of integers and store in IUSER */
  nag_convert_p2i(RHO, &E04VHF_IUSER[0]);
  nag_convert_p2i(USRFUN, &E04VHF_IUSER[2]);

  /* Call the NAG routine */
  IFAIL_integer = 1;
  e04vhf_(&START_integer, &NF_integer, &N_integer, &NXNAME_integer,
          &NFNAME_integer, &OBJADD_double, &OBJROW_integer, PROB_char,
          e04vhf_USRFUN,
          IAFUN_integerMatrix, JAVAR_integerMatrix, A_doubleMatrix,
          &LENA_integer,
          &NEA_integer, IGFUN_integerMatrix, JGVAR_integerMatrix,
          &LENG_integer,
          &NEG_integer, XLOW_doubleMatrix, XUPP_doubleMatrix, XNAMES_charArray,
          FLOW_doubleMatrix, FUPP_doubleMatrix, FNAMES_charArray,
          X_doubleMatrix,
          XSTATE_integerMatrix, XMUL_doubleMatrix, F_doubleMatrix,
          FSTATE_integerMatrix,
          FMUL_doubleMatrix, &NS_integer, &NINF_integer, &SINF_double,
          CW_charArray,
          &LENCW_integer, IW_integerMatrix, &LENIW_integer, RW_doubleMatrix,
          &LENRW_integer, CUSER_charArray, E04VHF_IUSER, E04VHF_RUSER,
          &IFAIL_integer,
          PROB_length, 8, 8, 8,
          8);

  /* Close the monitoring file if one opened */

  if (unit_num > 0)
    {
      IFAIL_integer = 1;
      x04adf_(&unit_num, &IFAIL_integer);

      /* Set the advisory channel back to its previous value */
      iflag = 1;
      x04abf_(&iflag, &nadv);
    }

  /* Error returns from NAG routine */
  if (IFAIL_integer != 0)
    {
      warning(
        "ABNORMAL EXIT from NAG library routine  E04VHF, : IFAIL = %ld\n",
        (long) IFAIL_integer);
    }

/* Get the list of results */
  PROTECT(ans = allocVector(VECSXP, 10));
  SET_VECTOR_ELT(ans, 0, nag_md2r(1, X_dim1, 1, X_doubleMatrix));
  SET_VECTOR_ELT(ans, 1, nag_mi2r(1, XSTATE_dim1, 1, XSTATE_integerMatrix));
  SET_VECTOR_ELT(ans, 2, nag_md2r(1, XMUL_dim1, 1, XMUL_doubleMatrix));
  SET_VECTOR_ELT(ans, 3, nag_md2r(1, F_dim1, 1, F_doubleMatrix));
  SET_VECTOR_ELT(ans, 4, nag_mi2r(1, FSTATE_dim1, 1, FSTATE_integerMatrix));
  SET_VECTOR_ELT(ans, 5, nag_md2r(1, FMUL_dim1, 1, FMUL_doubleMatrix));
  SET_VECTOR_ELT(ans, 6, nag_i2r(1, &NS_integer));
  SET_VECTOR_ELT(ans, 7, nag_i2r(1, &NINF_integer));
  SET_VECTOR_ELT(ans, 8, nag_d2r(1, &SINF_double));
  SET_VECTOR_ELT(ans, 9, nag_i2r(1, &IFAIL_integer));
  /* Get names for the list of results */
  PROTECT(names = allocVector(STRSXP, 10));
  SET_STRING_ELT(names, 0, mkChar("X"));
  SET_STRING_ELT(names, 1, mkChar("XSTATE"));
  SET_STRING_ELT(names, 2, mkChar("XMUL"));
  SET_STRING_ELT(names, 3, mkChar("F"));
  SET_STRING_ELT(names, 4, mkChar("FSTATE"));
  SET_STRING_ELT(names, 5, mkChar("FMUL"));
  SET_STRING_ELT(names, 6, mkChar("NS"));
  SET_STRING_ELT(names, 7, mkChar("NINF"));
  SET_STRING_ELT(names, 8, mkChar("SINF"));
  SET_STRING_ELT(names, 9, mkChar("IFAIL"));
  setAttrib(ans, R_NamesSymbol, names);
/* Unprotect the ans and names objects */
  UNPROTECT(2);

/*Call the clean-up routine*/
  E04VHF_clean_up(IAFUN_integerMatrix, JAVAR_integerMatrix, A_doubleMatrix,
                  IGFUN_integerMatrix, JGVAR_integerMatrix, XLOW_doubleMatrix,
                  XUPP_doubleMatrix, XNAMES_charArray, FLOW_doubleMatrix,
                  FUPP_doubleMatrix,
                  FNAMES_charArray, X_doubleMatrix, XSTATE_integerMatrix,
                  XMUL_doubleMatrix,
                  F_doubleMatrix, FSTATE_integerMatrix, FMUL_doubleMatrix,
                  CW_charArray,
                  IW_integerMatrix,
                  RW_doubleMatrix);

  return(ans);
}

void NAG_CALL e04vhf_USRFUN(Integer *STATUS, CONST Integer *N,
                            CONST double X[], CONST Integer *NEEDF,
                            CONST Integer *NF, double F[],
                            CONST Integer *NEEDG, CONST Integer *LENG,
                            double G[], char CUSER[],
                            Integer *E04VHF_IUSER, double E04VHF_RUSER[],
                            CONST int CUSER_dim)
{
  SEXP funct, rho, R_fcall, result;
/* Get the pointer to the R environment */
  nag_convert_i2p(&E04VHF_IUSER[0], &rho); /* Get the pointer to the R function */
  nag_convert_i2p(&E04VHF_IUSER[2], &funct);

/* Give the arguments to the function */
  PROTECT(R_fcall = lang10(
            funct,
            nag_fi2r(1, STATUS),
            nag_fi2r(1, N),
            nag_md2r(1, *N, 1, X),
            nag_fi2r(1, NEEDF),
            nag_fi2r(1, NF),
            nag_md2r(1, *NF, 1, F),
            nag_fi2r(1, NEEDG),
            nag_fi2r(1, LENG),
            nag_md2r(1, *LENG, 1, G)));

/* Get the results from evaluating the function in R */
  PROTECT(result = eval(R_fcall, rho));
/* put the results returned by function into C variables */
  (void) nag_getIntFromList("STATUS", result, STATUS);
  (void) nag_getDoubleMatrixFromList("F", 1, *NF, 1, result, F);
  (void) nag_getDoubleMatrixFromList("G", 1, *LENG, 1, result, G);

/* Release R_fcall */
  UNPROTECT(2);
}

int wE04VHF_handle_options(SEXP OPTLIST, char CW_charArray[],
                           Integer IW_integerMatrix[],
                           double RW_doubleMatrix[],
                           CONST int CW_dim)
{
  SEXP       names, tentry;
  char       err_msg[LEN_ERR_MSG];
  int        i, ival;
  Integer    IFAIL_setter_integer, STRING_length;
  double     dval;
  const char *this_name;
  char       STRING_char[100];
  int        unit_num = 0;

  /* No options specified */
  if (!isNewList(OPTLIST))
    {
      return 0;
    }

  /* Get the list of names from the list */
  PROTECT(names = getAttrib(OPTLIST, R_NamesSymbol));
  for (i = 0; i < length(OPTLIST); i++)
    {
      this_name = CHAR(STRING_ELT(names, i));

      /* Only interested in the first element, which is what asReal should
         convert to a double */
      PROTECT(tentry = VECTOR_ELT(OPTLIST, i));
      if (!isString(tentry))
        {
          dval = asReal(tentry);
          if (!R_FINITE(dval)) dval = 0.0;
          ival = (int) dval;
        }
      else
        {
          dval = -1.0;
          ival = -1;
        }

      STRING_char[0] = '\0';
      if (!strcmp(this_name, "list"))
        {
          if (ival)
            {
              sprintf(STRING_char, "list");
            }
          else
            {
              sprintf(STRING_char, "nolist");
            }
        }
      else if (!strcmp(this_name, "Central Difference Interval"))
        {
          sprintf(STRING_char, "Central Difference Interval = %g", dval);
        }
      else if (!strcmp(this_name, "Check Frequency"))
        {
          sprintf(STRING_char, "Check Frequency = %d", ival);
        }
      else if (!strcmp(this_name, "Crash Option"))
        {
          sprintf(STRING_char, "Crash Option = %d", ival);
        }
      else if (!strcmp(this_name, "Crash Tolerance"))
        {
          sprintf(STRING_char, "Crash Tolerance = %g", dval);
        }
      else if (!strcmp(this_name, "Derivative Option"))
        {
          sprintf(STRING_char, "Derivative Option = %d", ival);
        }
      else if (!strcmp(this_name, "Difference Interval"))
        {
          sprintf(STRING_char, "Difference Interval = %g", dval);
        }
      else if (!strcmp(this_name, "Elastic Weight"))
        {
          sprintf(STRING_char, "Elastic Weight = %g", dval);
        }
      else if (!strcmp(this_name, "Expand Frequency"))
        {
          sprintf(STRING_char, "Expand Frequency = %d", ival);
        }
      else if (!strcmp(this_name, "Factorization Frequency"))
        {
          sprintf(STRING_char, "Factorization Frequency = %d", ival);
        }
      else if (!strcmp(this_name, "Function Precision"))
        {
          sprintf(STRING_char, "Function Precision = %g", dval);
        }
      else if (!strcmp(this_name, "Hessian Frequency"))
        {
          sprintf(STRING_char, "Hessian Frequency = %d", ival);
        }
      else if (!strcmp(this_name, "Hessian Updates"))
        {
          sprintf(STRING_char, "Hessian Updates = %d", ival);
        }
      else if (!strcmp(this_name, "Infinite Bound Size"))
        {
          sprintf(STRING_char, "Infinite Bound Size = %g", dval);
        }
      else if (!strcmp(this_name, "Iterations Limit"))
        {
          sprintf(STRING_char, "Iterations Limit = %d", ival);
        }
      else if (!strcmp(this_name, "Linesearch Tolerance"))
        {
          sprintf(STRING_char, "Linesearch Tolerance = %g", dval);
        }
      else if (!strcmp(this_name, "Major Feasibility Tolerance"))
        {
          sprintf(STRING_char, "Major Feasibility Tolerance = %g", dval);
        }
      else if (!strcmp(this_name, "Major Optimality Tolerance"))
        {
          sprintf(STRING_char, "Major Optimality Tolerance = %g", dval);
        }
      else if (!strcmp(this_name, "Major Iterations Limit"))
        {
          sprintf(STRING_char, "Major Iterations Limit = %d", ival);
        }
      else if (!strcmp(this_name, "Major Print Level"))
        {
          sprintf(STRING_char, "Major Print Level = %d", ival);
        }
      else if (!strcmp(this_name, "Major Step Limit"))
        {
          sprintf(STRING_char, "Major Step Limit = %g", dval);
        }
      else if (!strcmp(this_name, "Minor Feasibility Tolerance"))
        {
          sprintf(STRING_char, "Minor Feasibility Tolerance = %g", dval);
        }
      else if (!strcmp(this_name, "Feasibility Tolerance"))
        {
          sprintf(STRING_char, "Feasibility Tolerance = %g", dval);
        }
      else if (!strcmp(this_name, "Minor Iterations Limit"))
        {
          sprintf(STRING_char, "Minor Iterations Limit = %d", ival);
        }
      else if (!strcmp(this_name, "Minor Print Level"))
        {
          sprintf(STRING_char, "Minor Print Level = %d", ival);
        }
      else if (!strcmp(this_name, "New Superbasics Limit"))
        {
          sprintf(STRING_char, "New Superbasics Limit = %d", ival);
        }
      else if (!strcmp(this_name, "Old Basis File"))
        {
          sprintf(STRING_char, "Old Basis File = %d", ival);
        }
      else if (!strcmp(this_name, "Partial Price"))
        {
          sprintf(STRING_char, "Partial Price = %d", ival);
        }
      else if (!strcmp(this_name, "Pivot Tolerance"))
        {
          sprintf(STRING_char, "Pivot Tolerance = %g", dval);
        }
      else if (!strcmp(this_name, "Print File"))
        {
          sprintf(STRING_char, "Print File = %d", ival);
        }
      else if (!strcmp(this_name, "Print Frequency"))
        {
          sprintf(STRING_char, "Print Frequency = %d", ival);
        }
      else if (!strcmp(this_name, "Proximal Point Method"))
        {
          sprintf(STRING_char, "Proximal Point Method = %d", ival);
        }
      else if (!strcmp(this_name, "Scale Option"))
        {
          sprintf(STRING_char, "Scale Option = %d", ival);
        }
      else if (!strcmp(this_name, "Scale Tolerance"))
        {
          sprintf(STRING_char, "Scale Tolerance = %g", dval);
        }
      else if (!strcmp(this_name, "Solution File"))
        {
          sprintf(STRING_char, "Solution File = %d", ival);
        }
      else if (!strcmp(this_name, "Superbasics Limit"))
        {
          sprintf(STRING_char, "Superbasics Limit = %d", ival);
        }
      else if (!strcmp(this_name, "Timing Level"))
        {
          sprintf(STRING_char, "Timing Level = %d", ival);
        }
      else if (!strcmp(this_name, "Verify Level"))
        {
          sprintf(STRING_char, "Verify Level = %d", ival);
        }
      else if (!strcmp(this_name, "Violation Limit"))
        {
          sprintf(STRING_char, "Violation Limit = %g", dval);
        }

      /* All other options are ignored (this includes the valid ones: cold start and warm start) */

      UNPROTECT(1);

      /* Call the option setting routine */
      if (STRING_char[0])
        {
          IFAIL_setter_integer = 1;
          STRING_length = strlen(STRING_char);
          e04vlf_(STRING_char, CW_charArray, IW_integerMatrix, RW_doubleMatrix,
                  &IFAIL_setter_integer, STRING_length,
                  8);

          if (IFAIL_setter_integer != 0)
            {
              sprintf(err_msg, "E04VLF failed with IFAIL = %ld\n",
                      (long) IFAIL_setter_integer);
              error(err_msg);
            }
        }
    }
  UNPROTECT(1);
  return unit_num;
}

void E04VHF_clean_up(Integer *IAFUN, Integer *JAVAR, double *A, Integer *IGFUN,
                     Integer *JGVAR, double *XLOW, double *XUPP, char *XNAMES,
                     double *FLOW,
                     double *FUPP, char *FNAMES, double *X, Integer *XSTATE,
                     double *XMUL, double *F,
                     Integer *FSTATE, double *FMUL, char *CW, Integer *IW,
                     double *RW)
{
  if (IAFUN) free(IAFUN);
  if (JAVAR) free(JAVAR);
  if (A) free(A);
  if (IGFUN) free(IGFUN);
  if (JGVAR) free(JGVAR);
  if (XLOW) free(XLOW);
  if (XUPP) free(XUPP);
  if (XNAMES) free(XNAMES);
  if (FLOW) free(FLOW);
  if (FUPP) free(FUPP);
  if (FNAMES) free(FNAMES);
  if (X) free(X);
  if (XSTATE) free(XSTATE);
  if (XMUL) free(XMUL);
  if (F) free(F);
  if (FSTATE) free(FSTATE);
  if (FMUL) free(FMUL);
  if (CW) free(CW);
  if (IW) free(IW);
  if (RW) free(RW);
  return;
}
