
#include <R.h>
#include <Rinternals.h>
#include "nag_rwrapper_utils.h"
#include "nagmk23.h"
#ifdef __cplusplus
extern "C" {
#endif
extern void NAG_CALL e04vjf_USRFUN(Integer *STATUS, CONST Integer *N,
                                   CONST double X[], CONST Integer *NEEDF,
                                   CONST Integer *NF, double F[],
                                   CONST Integer *NEEDG, CONST Integer *LENG,
                                   double G[], char CUSER[],
#if defined (_WIN32) && defined (USE_STDCALL)
                                   CONST int CUSER_dim,
#endif
                                   Integer *E04VJF_IUSER,
                                   double E04VJF_RUSER[]
#if defined (_WIN32) && defined (USE_STDCALL)
#else
                                  , CONST int CUSER_dim
#endif
                                   );

#ifdef __cplusplus
}
#endif

/* Length of array to hold error messages, no checks are made to ensure this is large enough */
#define LEN_ERR_MSG 100

void E04VJF_clean_up(Integer *IAFUN, Integer *JAVAR, double *A, Integer *IGFUN,
                     Integer *JGVAR, double *X, double *XLOW, double *XUPP,
                     char *CW, Integer *IW,
                     double *RW);

SEXP WE04VJF(SEXP NF, SEXP N, SEXP USRFUN, SEXP LENA, SEXP LENG, SEXP X,
             SEXP XLOW, SEXP XUPP,
             SEXP RHO)
{
  Integer IFAIL_integer, IFAIL_EXTRACTION_integer;
  Integer E04VJF_IUSER[4];
  double  E04VJF_RUSER[1];
  char    CUSER_charArray[1];
  SEXP    ans, names;

  /* Initialize the option arrays */
  Integer IFAIL_initializer_integer = 1;
  char    err_msg[LEN_ERR_MSG];

/*Declare dimension variables for option initializer*/
  Integer LENCW_integer = 600;
  Integer LENIW_integer = 600;
  Integer LENRW_integer = 600;

/*Declare the parameters for the option initializer*/
  char    *CW_charArray = 0;
  Integer *IW_integerMatrix = 0;
  double  *RW_doubleMatrix = 0;
  Integer CW_dim1 = LENCW_integer;
  if (!(CW_charArray = (char *) malloc(CW_dim1*8*sizeof(char))))
    {
      error("Memory allocation error for %s", "CW");
    }
  Integer IW_dim1 = LENIW_integer;
  if (!(IW_integerMatrix = (Integer *) malloc(IW_dim1*sizeof(Integer))))
    {
      error("Memory allocation error for %s", "IW");
    }
  Integer RW_dim1 = LENRW_integer;
  if (!(RW_doubleMatrix = (double *) malloc(RW_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "RW");
    }

  nag_initializeIntegerMatrix(IW_dim1, 1, IW_integerMatrix);
  nag_initializeDoubleMatrix(RW_dim1, 1, RW_doubleMatrix);
  e04vgf_(CW_charArray, &LENCW_integer, IW_integerMatrix, &LENIW_integer,
          RW_doubleMatrix, &LENRW_integer, &IFAIL_initializer_integer,
          8);

  if (IFAIL_initializer_integer != 0)
    {
      sprintf(err_msg, "E04VGF failed with IFAIL = %ld\n",
              (long) IFAIL_initializer_integer);
      error(err_msg);
    }
/*initialization for input variables*/
  int     NF_integer;
  if (nag_extractInteger(NF, &NF_integer))
    {
      error("Integer extraction error for %s", "NF");
    }
  int     N_integer;
  if (nag_extractInteger(N, &N_integer))
    {
      error("Integer extraction error for %s", "N");
    }
  int     LENA_integer;
  if (nag_extractInteger(LENA, &LENA_integer))
    {
      error("Integer extraction error for %s", "LENA");
    }
  int     LENG_integer;
  if (nag_extractInteger(LENG, &LENG_integer))
    {
      error("Integer extraction error for %s", "LENG");
    }
  double  *X_doubleMatrix = 0;
  double  *XLOW_doubleMatrix = 0;
  double  *XUPP_doubleMatrix = 0;

/*initialization for output variables*/
  Integer *IAFUN_integerMatrix = 0;
  Integer *JAVAR_integerMatrix = 0;
  double  *A_doubleMatrix = 0;
  int     NEA_integer;
  Integer *IGFUN_integerMatrix = 0;
  Integer *JGVAR_integerMatrix = 0;
  int     NEG_integer;

/*allocation general variables*/
  Integer IAFUN_dim1 = LENA_integer;
  if (!(IAFUN_integerMatrix = (Integer *) malloc(IAFUN_dim1*sizeof(Integer))))
    {
      error("Memory allocation error for %s", "IAFUN");
    }
  Integer JAVAR_dim1 = LENA_integer;
  if (!(JAVAR_integerMatrix = (Integer *) malloc(JAVAR_dim1*sizeof(Integer))))
    {
      error("Memory allocation error for %s", "JAVAR");
    }
  Integer A_dim1 = LENA_integer;
  if (!(A_doubleMatrix = (double *) malloc(A_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "A");
    }
  Integer IGFUN_dim1 = LENG_integer;
  if (!(IGFUN_integerMatrix = (Integer *) malloc(IGFUN_dim1*sizeof(Integer))))
    {
      error("Memory allocation error for %s", "IGFUN");
    }
  Integer JGVAR_dim1 = LENG_integer;
  if (!(JGVAR_integerMatrix = (Integer *) malloc(JGVAR_dim1*sizeof(Integer))))
    {
      error("Memory allocation error for %s", "JGVAR");
    }
  Integer X_dim1 = N_integer;
  if (!(X_doubleMatrix = (double *) malloc(X_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "X");
    }
  Integer XLOW_dim1 = N_integer;
  if (!(XLOW_doubleMatrix = (double *) malloc(XLOW_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "XLOW");
    }
  Integer XUPP_dim1 = N_integer;
  if (!(XUPP_doubleMatrix = (double *) malloc(XUPP_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "XUPP");
    }

/*allocation for dimension variables*/

/*extraction for input variables*/
  if ((IFAIL_EXTRACTION_integer =
         nag_extractDoubleMatrix(1, X_dim1, 1, X, X_doubleMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "X", nrows(X), X_dim1);
        }
      else
        {
          E04VJF_clean_up(IAFUN_integerMatrix, JAVAR_integerMatrix,
                          A_doubleMatrix, IGFUN_integerMatrix,
                          JGVAR_integerMatrix, X_doubleMatrix,
                          XLOW_doubleMatrix, XUPP_doubleMatrix, CW_charArray,
                          IW_integerMatrix,
                          RW_doubleMatrix);
          error("Array extraction error for %s", "X");
        }
    }
  if ((IFAIL_EXTRACTION_integer =
         nag_extractDoubleMatrix(1, XLOW_dim1, 1, XLOW, XLOW_doubleMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "XLOW", nrows(XLOW), XLOW_dim1);
        }
      else
        {
          E04VJF_clean_up(IAFUN_integerMatrix, JAVAR_integerMatrix,
                          A_doubleMatrix, IGFUN_integerMatrix,
                          JGVAR_integerMatrix, X_doubleMatrix,
                          XLOW_doubleMatrix, XUPP_doubleMatrix, CW_charArray,
                          IW_integerMatrix,
                          RW_doubleMatrix);
          error("Array extraction error for %s", "XLOW");
        }
    }
  if ((IFAIL_EXTRACTION_integer =
         nag_extractDoubleMatrix(1, XUPP_dim1, 1, XUPP, XUPP_doubleMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "XUPP", nrows(XUPP), XUPP_dim1);
        }
      else
        {
          E04VJF_clean_up(IAFUN_integerMatrix, JAVAR_integerMatrix,
                          A_doubleMatrix, IGFUN_integerMatrix,
                          JGVAR_integerMatrix, X_doubleMatrix,
                          XLOW_doubleMatrix, XUPP_doubleMatrix, CW_charArray,
                          IW_integerMatrix,
                          RW_doubleMatrix);
          error("Array extraction error for %s", "XUPP");
        }
    }

/* Perform some initial checks */
  if (!isEnvironment(RHO))
    {
      error("rho must be an environment");
    }

  /* Convert function pointer funct and rho into a couple of integers and store in IUSER */
  nag_convert_p2i(RHO, &E04VJF_IUSER[0]);
  nag_convert_p2i(USRFUN, &E04VJF_IUSER[2]);

  /* Call the NAG routine */
  IFAIL_integer = 1;
  e04vjf_(&NF_integer, &N_integer, e04vjf_USRFUN, IAFUN_integerMatrix,
          JAVAR_integerMatrix, A_doubleMatrix, &LENA_integer, &NEA_integer,
          IGFUN_integerMatrix, JGVAR_integerMatrix, &LENG_integer,
          &NEG_integer,
          X_doubleMatrix, XLOW_doubleMatrix, XUPP_doubleMatrix, CW_charArray,
          &LENCW_integer, IW_integerMatrix, &LENIW_integer, RW_doubleMatrix,
          &LENRW_integer, CUSER_charArray, E04VJF_IUSER, E04VJF_RUSER,
          &IFAIL_integer,
          8,
          8);

  /* Error returns from NAG routine */
  if (IFAIL_integer != 0)
    {
      warning(
        "ABNORMAL EXIT from NAG library routine  E04VJF, : IFAIL = %ld\n",
        (long) IFAIL_integer);
    }

/* Get the list of results */
  PROTECT(ans = allocVector(VECSXP, 8));
  SET_VECTOR_ELT(ans, 0, nag_mi2r(1, IAFUN_dim1, 1, IAFUN_integerMatrix));
  SET_VECTOR_ELT(ans, 1, nag_mi2r(1, JAVAR_dim1, 1, JAVAR_integerMatrix));
  SET_VECTOR_ELT(ans, 2, nag_i2r(1, &NEA_integer));
  SET_VECTOR_ELT(ans, 3, nag_md2r(1, A_dim1, 1, A_doubleMatrix));
  SET_VECTOR_ELT(ans, 4, nag_mi2r(1, IGFUN_dim1, 1, IGFUN_integerMatrix));
  SET_VECTOR_ELT(ans, 5, nag_mi2r(1, JGVAR_dim1, 1, JGVAR_integerMatrix));
  SET_VECTOR_ELT(ans, 6, nag_i2r(1, &NEG_integer));
  SET_VECTOR_ELT(ans, 7, nag_i2r(1, &IFAIL_integer));
  /* Get names for the list of results */
  PROTECT(names = allocVector(STRSXP, 8));
  SET_STRING_ELT(names, 0, mkChar("IAFUN"));
  SET_STRING_ELT(names, 1, mkChar("JAVAR"));
  SET_STRING_ELT(names, 2, mkChar("NEA"));
  SET_STRING_ELT(names, 3, mkChar("A"));
  SET_STRING_ELT(names, 4, mkChar("IGFUN"));
  SET_STRING_ELT(names, 5, mkChar("JGVAR"));
  SET_STRING_ELT(names, 6, mkChar("NEG"));
  SET_STRING_ELT(names, 7, mkChar("IFAIL"));
  setAttrib(ans, R_NamesSymbol, names);
/* Unprotect the ans and names objects */
  UNPROTECT(2);

/*Call the clean-up routine*/
  E04VJF_clean_up(IAFUN_integerMatrix, JAVAR_integerMatrix, A_doubleMatrix,
                  IGFUN_integerMatrix, JGVAR_integerMatrix, X_doubleMatrix,
                  XLOW_doubleMatrix,
                  XUPP_doubleMatrix, CW_charArray, IW_integerMatrix,
                  RW_doubleMatrix);

  return(ans);
}

void NAG_CALL e04vjf_USRFUN(Integer *STATUS, CONST Integer *N,
                            CONST double X[], CONST Integer *NEEDF,
                            CONST Integer *NF, double F[],
                            CONST Integer *NEEDG, CONST Integer *LENG,
                            double G[], char CUSER[],
#if defined (_WIN32) && defined (USE_STDCALL)
                            CONST int CUSER_dim,
#endif
                            Integer *E04VJF_IUSER, double E04VJF_RUSER[]

        #if defined (_WIN32) && defined (USE_STDCALL)
#else
                           , CONST int CUSER_dim
#endif
                            )
{
  SEXP funct, rho, R_fcall, result;
/* Get the pointer to the R environment */
  nag_convert_i2p(&E04VJF_IUSER[0], &rho); /* Get the pointer to the R function */
  nag_convert_i2p(&E04VJF_IUSER[2], &funct);

/* Give the arguments to the function */
  PROTECT(R_fcall = lang10(
            funct,
            nag_fi2r(1, STATUS),
            nag_fi2r(1, N),
            nag_md2r(1, *N, 1, X),
            nag_fi2r(1, NEEDF),
            nag_fi2r(1, NF),
            nag_md2r(1, *NF, 1, F),
            nag_fi2r(1, NEEDG),
            nag_fi2r(1, LENG),
            nag_md2r(1, *LENG, 1, G)));

/* Get the results from evaluating the function in R */
  PROTECT(result = eval(R_fcall, rho));
/* put the results returned by function into C variables */
  (void) nag_getIntFromList("STATUS", result, STATUS);
  (void) nag_getDoubleMatrixFromList("F", 1, *NF, 1, result, F);
  (void) nag_getDoubleMatrixFromList("G", 1, *LENG, 1, result, G);

/* Release R_fcall */
  UNPROTECT(2);
}

void E04VJF_clean_up(Integer *IAFUN, Integer *JAVAR, double *A, Integer *IGFUN,
                     Integer *JGVAR, double *X, double *XLOW, double *XUPP,
                     char *CW, Integer *IW,
                     double *RW)
{
  if (IAFUN) free(IAFUN);
  if (JAVAR) free(JAVAR);
  if (A) free(A);
  if (IGFUN) free(IGFUN);
  if (JGVAR) free(JGVAR);
  if (X) free(X);
  if (XLOW) free(XLOW);
  if (XUPP) free(XUPP);
  if (CW) free(CW);
  if (IW) free(IW);
  if (RW) free(RW);
  return;
}
