
#include <R.h>
#include <Rinternals.h>
#include "nag_rwrapper_utils.h"
#include "nagmk23.h"
#ifdef __cplusplus
extern "C" {
#endif

#ifdef __cplusplus
}
#endif

/* Length of array to hold error messages, no checks are made to ensure this is large enough */
#define LEN_ERR_MSG 100

void S18GKF_clean_up(Complex *B);

SEXP WS18GKF(SEXP Z, SEXP A, SEXP NL)
{
  Integer IFAIL_integer;
  SEXP    ans, names;

/*initialization for input variables*/
  Complex Z_complex;
  if (nag_extractComplex(Z, &Z_complex))
    {
      error("Complex extraction error for %s", "Z");
    }
  double  A_double;
  if (nag_extractDouble(A, &A_double))
    {
      error("Double extraction error for %s", "A");
    }
  int     NL_integer;
  if (nag_extractInteger(NL, &NL_integer))
    {
      error("Integer extraction error for %s", "NL");
    }

/*initialization for output variables*/
  Complex *B = 0;

/*allocation general variables*/
  Integer B_dim1 = abs(NL_integer)+1;
  if (!(B = (Complex *) malloc(B_dim1*sizeof(Complex))))
    {
      error("Memory allocation error for %s", "B");
    }

/*extraction for input variables*/

  /* Call the NAG routine */
  IFAIL_integer = 1;
  s18gkf_(&Z_complex, &A_double, &NL_integer, B, &IFAIL_integer);

  /* Error returns from NAG routine */
  if (IFAIL_integer != 0)
    {
      warning(
        "ABNORMAL EXIT from NAG library routine  S18GKF, : IFAIL = %ld\n",
        (long) IFAIL_integer);
    }

/* Get the list of results */
  PROTECT(ans = allocVector(VECSXP, 2));
  SET_VECTOR_ELT(ans, 0, nag_cplx2r(B_dim1, B));
  SET_VECTOR_ELT(ans, 1, nag_i2r(1, &IFAIL_integer));
  /* Get names for the list of results */
  PROTECT(names = allocVector(STRSXP, 2));
  SET_STRING_ELT(names, 0, mkChar("B"));
  SET_STRING_ELT(names, 1, mkChar("IFAIL"));
  setAttrib(ans, R_NamesSymbol, names);
/* Unprotect the ans and names objects */
  UNPROTECT(2);

/*Call the clean-up routine*/
  S18GKF_clean_up(B);

  return(ans);
}

void S18GKF_clean_up(Complex *B)
{
  if (B) free(B);
  return;
}
